@@ -13,10 +13,17 @@ log = Logger("opengl")
 required_extensions = ["GL_ARB_texture_rectangle", "GL_ARB_vertex_program"]
 
 
+WHITELIST = {}
 BLACKLIST = {"vendor" : ["nouveau", "Humper", "VMware, Inc."]}
-if sys.platform.startswith("darwin"):
-    #crashes were reported with the Intel driver on OSX
+if False:
+    #for testing:
+    BLACKLIST["vendor"].append("NVIDIA Corporation")
+    WHITELIST["renderer"] = ["GeForce GTX 760/PCIe/SSE2"]
+    
+if sys.platform.startswith("darwin"): 
+    #crashes were reported with the Intel driver on OSX 
     BLACKLIST["vendor"].append("Intel Inc.")
+    WHITELIST["renderer"] = ["Intel HD Graphics 4000 OpenGL Engine"]
 
 
 DEFAULT_ALPHA = not sys.platform.startswith("win") and not sys.platform.startswith("darwin")
@@ -170,7 +177,7 @@ def check_GL_support(gldrawable, glcontext, min_texture_size=0, force_enable=Fal
                 if fatal:
                     gl_check_error("OpenGL property '%s' is missing" % d)
                 else:
-                    log.warn("OpenGL property '%s' is missing" % d)
+                    log("OpenGL property '%s' is missing", d)
                 v = ""
             props[d] = v
 
@@ -180,13 +187,25 @@ def check_GL_support(gldrawable, glcontext, min_texture_size=0, force_enable=Fal
             log("%s: %s", d, v)
             props[d] = v
 
+        blacklisted = None
+        whitelisted = None
         for k,vlist in BLACKLIST.items():
             v = props.get(k)
             if v in vlist:
-                if force_enable:
-                    log.warn("Warning: %s '%s' is blacklisted!", k, v)
-                else:
-                    gl_check_error("%s '%s' is blacklisted!" % (k, v))
+                log("%s '%s' found in blacklist: %s", k, v, vlist)
+                blacklisted = k, v
+        for k,vlist in WHITELIST.items():
+            v = props.get(k)
+            if v in vlist:
+                log("%s '%s' found in whitelist: %s", k, v, vlist)
+                whitelisted = k, v
+        if blacklisted:
+            if whitelisted:
+                log.info("%s '%s' enabled (found in both blacklist and whitelist)", *whitelisted)
+            elif force_enable:
+                log.warn("Warning: %s '%s' is blacklisted!", *blacklisted)
+            else:
+                gl_check_error("%s '%s' is blacklisted!" % (blacklisted))
 
         #check for specific functions we need:
         from OpenGL.GL import glActiveTexture, glTexSubImage2D, glTexCoord2i, \
@@ -329,16 +348,11 @@ def check_GL_support(gldrawable, glcontext, min_texture_size=0, force_enable=Fal
         gldrawable.gl_end()
 
 def check_support(min_texture_size=0, force_enable=False, check_colormap=False):
-    try:
-        from xpra.platform.paths import get_icon_dir
-        opengl_icon = os.path.join(get_icon_dir(), "opengl.png")
-    except:
-        opengl_icon = None
-    #tricks to get py2exe to include what we need / load it from its unusual path:
-    if sys.platform.startswith("win"):
-        #This is supposed to help py2exe
-        #(must be done after we setup the sys.path in platform.win32.paths):
-        from OpenGL.platform import win32   #@UnusedImport
+    #platform checks:
+    from xpra.platform.gui import gl_check
+    warning = gl_check()
+    if warning:
+        gl_check_error(warning)
 
     props = {}
     import gtk.gdk
@@ -376,14 +390,6 @@ def check_support(min_texture_size=0, force_enable=False, check_colormap=False):
         w = gtk.Window()
         w.set_decorated(False)
         vbox = gtk.VBox()
-        width, height = 32, 32
-        if opengl_icon and os.path.exists(opengl_icon) and hasattr(gtk, "image_new_from_pixbuf"):
-            pixbuf = gtk.gdk.pixbuf_new_from_file(opengl_icon)
-            image = gtk.image_new_from_pixbuf(pixbuf)
-            vbox.add(image)
-            width, height = pixbuf.get_width(), pixbuf.get_height()
-            w.set_default_size(width, height)
-            w.set_resizable(False)
         glarea = gtk.gtkgl.DrawingArea(glconfig)
         glarea.set_size_request(32, 32)
         vbox.add(glarea)
