@@ -5,6 +5,7 @@
 # later version. See the file COPYING for details.
 
 #only works with gtk2:
+import sys
 import os
 from gtk import gdk
 assert gdk
@@ -16,12 +17,13 @@ from xpra.log import Logger
 log = Logger("opengl", "paint")
 OPENGL_DEBUG = os.environ.get("XPRA_OPENGL_DEBUG", "0")=="1"
 
-
 from xpra.codecs.codec_constants import get_subsampling_divs
-from xpra.client.gl.gl_check import get_DISPLAY_MODE
+from xpra.client.gl.gl_check import get_DISPLAY_MODE, GL_ALPHA_SUPPORTED
 from xpra.client.gl.gl_colorspace_conversions import YUV2RGB_shader, RGBP2RGB_shader
 from xpra.client.gtk2.window_backing import GTK2WindowBacking, fire_paint_callbacks
-from OpenGL.GL import GL_PROJECTION, GL_MODELVIEW, \
+from OpenGL import version as OpenGL_version
+from OpenGL.GL import \
+    GL_PROJECTION, GL_MODELVIEW, \
     GL_UNPACK_ROW_LENGTH, GL_UNPACK_ALIGNMENT, \
     GL_TEXTURE_MAG_FILTER, GL_TEXTURE_MIN_FILTER, GL_NEAREST, \
     GL_UNSIGNED_BYTE, GL_LUMINANCE, GL_LINEAR, \
@@ -47,6 +49,7 @@ from OpenGL.GL.ARB.vertex_program import glGenProgramsARB, \
 from OpenGL.GL.ARB.fragment_program import GL_FRAGMENT_PROGRAM_ARB
 from OpenGL.GL.ARB.framebuffer_object import GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, glGenFramebuffers, glBindFramebuffer, glFramebufferTexture2D
 
+from ctypes import c_uint
 
 PIXEL_FORMAT_TO_CONSTANT = {
                        "BGR"    : GL_BGR,
@@ -97,6 +100,12 @@ if OPENGL_DEBUG:
 from ctypes import c_char_p
 
 
+#support for memory views requires Python 2.7 and PyOpenGL 3.1
+memoryview_type = None
+if sys.version_info[:2]>=(2,7) and OpenGL_version.__version__.split('.')[:2]>=['3','1']:
+    memoryview_type = memoryview
+
+
 # Texture number assignment
 #  1 = Y plane
 #  2 = U plane
@@ -127,28 +136,40 @@ window image, which is critical because of backbuffer content losses upon buffer
 """
 class GLPixmapBacking(GTK2WindowBacking):
 
-    def __init__(self, wid, w, h, has_alpha):
-        GTK2WindowBacking.__init__(self, wid, w, h, has_alpha)
-        display_mode = get_DISPLAY_MODE()
+    RGB_MODES = ["YUV420P", "YUV422P", "YUV444P", "GBRP", "BGRA", "BGRX", "RGBA", "RGBX", "RGB", "BGR"]
+    HAS_ALPHA = GL_ALPHA_SUPPORTED
+
+    def __init__(self, wid, w, h, window_alpha):
+        alpha = GL_ALPHA_SUPPORTED and window_alpha
+        display_mode = get_DISPLAY_MODE(want_alpha=alpha)
         try:
             self.glconfig = gtk.gdkgl.Config(mode=display_mode)
         except gtk.gdkgl.NoMatches:
+            #toggle double buffering and try again:
             display_mode &= ~gtk.gdkgl.MODE_DOUBLE
+            log.warn("failed to initialize gl context: trying again %s double buffering", (bool(display_mode & gtk.gdkgl.MODE_DOUBLE) and "with") or "without")
             self.glconfig = gtk.gdkgl.Config(mode=display_mode)
+        GTK2WindowBacking.__init__(self, wid, alpha and self.glconfig.has_alpha())
         self._backing = gtk.gtkgl.DrawingArea(self.glconfig)
         #restoring missed masks:
         self._backing.set_events(self._backing.get_events() | gdk.POINTER_MOTION_MASK | gdk.POINTER_MOTION_HINT_MASK)
-        if self._has_alpha:
+        if self._alpha_enabled:
+            assert GL_ALPHA_SUPPORTED, "BUG: cannot enable alpha if GL backing does not support it!"
             screen = self._backing.get_screen()
             rgba = screen.get_rgba_colormap()
-            if rgba:
+            display = screen.get_display()
+            if not display.supports_composite():
+                log.warn("display %s does not support compositing, transparency disabled", display.get_name())
+                self._alpha_enabled = False
+            elif rgba:
+                log("%s.__init__() using rgba colormap %s", rgba)
                 self._backing.set_colormap(rgba)
             else:
                 log.warn("failed to enable transparency on screen %s", screen)
-                self._has_alpha = False
+                self._alpha_enabled = False
         #this is how many bpp we keep in the texture
         #(pixels are always stored in 32bpp - but this makes it clearer when we do/don't support alpha)
-        if self._has_alpha:
+        if self._alpha_enabled:
             self.texture_pixel_format = GL_RGBA
         else:
             self.texture_pixel_format = GL_RGB
@@ -224,7 +245,12 @@ class GLPixmapBacking(GTK2WindowBacking):
         assert self.offscreen_fbo is None
         assert self.shaders is None
         self.textures = glGenTextures(5)
-        self.offscreen_fbo = glGenFramebuffers(1)
+        if hasattr(glGenFramebuffers, "pyConverters") and len(glGenFramebuffers.pyConverters)==1:
+            #single argument syntax:
+            self.offscreen_fbo = glGenFramebuffers(1)
+        else:
+            self.offscreen_fbo = c_uint(1)
+            glGenFramebuffers(1, self.offscreen_fbo)
         log("%s.gl_init_textures() textures: %s, offscreen fbo: %s", self, self.textures, self.offscreen_fbo)
 
     def gl_init_shaders(self):
@@ -359,7 +385,7 @@ class GLPixmapBacking(GTK2WindowBacking):
         # Change state to target screen instead of our FBO
         glBindFramebuffer(GL_FRAMEBUFFER, 0)
 
-        if self._has_alpha:
+        if self._alpha_enabled:
             # transparent background:
             glClearColor(0.0, 0.0, 0.0, 0.0)
         else:
@@ -371,7 +397,7 @@ class GLPixmapBacking(GTK2WindowBacking):
         w, h = self.size
 
         glBindTexture(GL_TEXTURE_RECTANGLE_ARB, self.textures[TEX_FBO])
-        if self._has_alpha:
+        if self._alpha_enabled:
             # support alpha channel if present:
             glEnablei(GL_BLEND, self.textures[TEX_FBO])
             glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
@@ -423,23 +449,42 @@ class GLPixmapBacking(GTK2WindowBacking):
         finally:
             drawable.gl_end()
 
-    def _do_paint_rgb32(self, img_data, x, y, width, height, rowstride, options, callbacks):
-        return self._do_paint_rgb(32, img_data, x, y, width, height, rowstride, options, callbacks)
+    def _do_paint_rgb32(self, img_data, x, y, width, height, rowstride, options):
+        return self._do_paint_rgb(32, img_data, x, y, width, height, rowstride, options)
 
-    def _do_paint_rgb24(self, img_data, x, y, width, height, rowstride, options, callbacks):
-        return self._do_paint_rgb(24, img_data, x, y, width, height, rowstride, options, callbacks)
+    def _do_paint_rgb24(self, img_data, x, y, width, height, rowstride, options):
+        return self._do_paint_rgb(24, img_data, x, y, width, height, rowstride, options)
 
-    def _do_paint_rgb(self, bpp, img_data, x, y, width, height, rowstride, options, callbacks):
+    def _do_paint_rgb(self, bpp, img_data, x, y, width, height, rowstride, options):
         log("%s._do_paint_rgb(%s, %s bytes, x=%d, y=%d, width=%d, height=%d, rowstride=%d)", self, bpp, len(img_data), x, y, width, height, rowstride)
         drawable = self.gl_init()
         if not drawable:
             log("%s._do_paint_rgb(..) drawable is not set!", self)
             return False
 
+        #deal with buffers uploads by wrapping them if we can, or copy to a string:
+        if type(img_data)==buffer:
+            if memoryview_type is not None:
+                img_data = memoryview_type(img_data)
+            else:
+                img_data = str(img_data)
+
         try:
             self.set_rgb_paint_state()
 
-            bytes_per_pixel = bpp/8
+            rgb_format = options.get("rgb_format")
+            if not rgb_format:
+                #Older servers may not tell us the pixel format, so we must infer it:
+                if bpp==24:
+                    rgb_format = "RGB"
+                else:
+                    assert bpp==32
+                    rgb_format = "RGBA"
+            #convert it to a GL constant:
+            pformat = PIXEL_FORMAT_TO_CONSTANT.get(rgb_format)
+            assert pformat is not None, "could not find pixel format for %s (bpp=%s)" % (rgb_format, bpp)
+
+            bytes_per_pixel = len(rgb_format)       #ie: BGRX -> 4
             # Compute alignment and row length
             row_length = 0
             alignment = 1
@@ -453,17 +498,7 @@ class GLPixmapBacking(GTK2WindowBacking):
             if (rowstride - width * bytes_per_pixel) >= alignment:
                 row_length = width + (rowstride - width * bytes_per_pixel) / bytes_per_pixel
 
-            rgb_format = options.get("rgb_format", None)
-            self.gl_marker("%s %sbpp update at %d,%d, size %d,%d, stride is %d, row length %d, alignment %d" % (rgb_format, bpp, x, y, width, height, rowstride, row_length, alignment))
-            #Older clients may not tell us the pixel format, so we must infer it:
-            if bpp==24:
-                default_format = "RGB"
-            else:
-                assert bpp==32
-                default_format = "RGBA"
-            #convert it to a GL constant:
-            pformat = PIXEL_FORMAT_TO_CONSTANT.get(rgb_format or default_format)
-            assert pformat is not None, "could not find pixel format for %s or %s (bpp=%s)" % (rgb_format, default_format, bpp)
+            self.gl_marker("%s %sbpp update at (%d,%d) size %dx%d (%s bytes), stride=%d, row length %d, alignment %d, using GL upload format=%s" % (rgb_format, bpp, x, y, width, height, len(img_data), rowstride, row_length, alignment, CONSTANT_TO_PIXEL_FORMAT.get(pformat)))
 
             # Upload data as temporary RGB texture
             glBindTexture(GL_TEXTURE_RECTANGLE_ARB, self.textures[TEX_RGB])
@@ -496,16 +531,33 @@ class GLPixmapBacking(GTK2WindowBacking):
         return True
 
     def do_video_paint(self, img, x, y, enc_width, enc_height, width, height, options, callbacks):
-        #note: we clone here unconditionally because this gives us the pixels
-        #as a string object which PyOpenGL can use for upload, whereas the buffer objects
-        #we get from the avcodec and vpx decoders cannot be used.
-        #PyOpenGL version 3.1 has some support for the memory view interface
-        #but this requires memoryview support which requires python>=2.7
-        img.clone_pixel_data()
+        clone = True
+        #we can only use zero copy upload if the image is "thread_safe"
+        #(dec_avcodec2 is not safe...)
+        #and if the data is a memoryview (or if we can wrap it in one)
+        if memoryview_type is not None and img.is_thread_safe():
+            pixels = img.get_pixels()
+            assert len(pixels)==3, "invalid number of planes: %s" % len(pixels)
+            plane_types = list(set([type(plane) for plane in img.get_pixels()]))
+            if len(plane_types)==1:
+                plane_type = plane_types[0]
+                if plane_type==memoryview_type:
+                    clone = False
+                elif plane_type in (str, buffer):
+                    #wrap with a memoryview that pyopengl can use:
+                    views = []
+                    for i in range(3):
+                        views.append(memoryview_type(pixels[i]))
+                    img.set_pixels(views)
+                    clone = False
+        if clone:
+            #copy so the data will be usable (usually a str)
+            img.clone_pixel_data()
         gobject.idle_add(self.gl_paint_planar, img, x, y, enc_width, enc_height, width, height, callbacks)
 
     def gl_paint_planar(self, img, x, y, enc_width, enc_height, width, height, callbacks):
         #this function runs in the UI thread, no video_decoder lock held
+        log("gl_paint_planar%s", (img, x, y, enc_width, enc_height, width, height, callbacks))
         try:
             pixel_format = img.get_pixel_format()
             assert pixel_format in ("YUV420P", "YUV422P", "YUV444P", "GBRP"), "sorry the GL backing does not handle pixel format '%s' yet!" % (pixel_format)
@@ -516,6 +568,8 @@ class GLPixmapBacking(GTK2WindowBacking):
                 return
             try:
                 self.update_planar_textures(x, y, enc_width, enc_height, img, pixel_format, scaling=(enc_width!=width or enc_height!=height))
+                img.free()
+
                 # Update FBO texture
                 x_scale, y_scale = 1, 1
                 if width!=enc_width or height!=enc_height:
