@@ -26,6 +26,7 @@ from xpra.client.gtk_base.gtk_client_base import GTKXpraClient, xor_str
 from xpra.client.gtk2.tray_menu import GTK2TrayMenu
 from xpra.gtk_common.cursor_names import cursor_names
 from xpra.client.window_border import WindowBorder
+from xpra.net.compression import Uncompressed
 from xpra.log import Logger
 
 log = Logger("gtk", "client")
@@ -64,33 +65,52 @@ class XpraClient(GTKXpraClient):
 
     def init(self, opts):
         GTKXpraClient.init(self, opts)
+        self.ClientWindowClass = None
         if opts.window_layout:
             assert opts.window_layout in WINDOW_LAYOUTS
             self.ClientWindowClass = WINDOW_LAYOUTS.get(opts.window_layout)
+        if self.ClientWindowClass:
+            log.info("window layout '%s' specified, disabling OpenGL", opts.window_layout)
+            opts.opengl= False
         else:
             self.ClientWindowClass = BorderClientWindow
-        log("init(..) ClientWindowClass=%s", self.ClientWindowClass)
+            log("init(..) ClientWindowClass=%s", self.ClientWindowClass)
 
 
-    def parse_border(self, border_str):
+    def parse_border(self, border_str, extra_args):
         parts = [x.strip() for x in border_str.split(",")]
         color_str = parts[0]
+        def border_help():
+            log.info(" border format: color[,size]")
+            log.info("  eg: red,10")
+            log.info("  eg: ,5")
+            log.info("  eg: auto,5")
+            log.info("  eg: blue")
         if color_str.lower()=="none":
             return
-        if color_str=="auto":
+        if color_str.lower()=="help":
+            border_help()
+            return
+        if color_str=="auto" or color_str=="":
             try:
-                import hashlib
-                m = hashlib.sha1()
-                m.update(self._protocol._conn.target)
+                try: 
+                    from hashlib import md5
+                except ImportError:
+                    from md5 import md5
+                m = md5()
+                for x in extra_args:
+                    m.update(str(x))
                 color_str = "#%s" % m.hexdigest()[:6]
-                log("border color derived from %s: %s", self._protocol._conn.target, color_str)
+                log("border color derived from %s: %s", extra_args, color_str)
             except:
+                log.info("failed to derive border color from %s", extra_args, exc_info=True)
                 #fail: default to red
                 color_str = "red"
         try:
             color = gtk.gdk.color_parse(color_str)
         except Exception, e:
-            log.warn("invalid color specified: %s (%s)", color_str, e)
+            log.warn("invalid border color specified: '%s' (%s)", color_str, e)
+            border_help()
             color = gtk.gdk.Color("red")
         alpha = 0.6
         size = 4
@@ -142,12 +162,14 @@ class XpraClient(GTKXpraClient):
     def do_get_core_encodings(self):
         encodings = GTKXpraClient.do_get_core_encodings(self)
         #we can handle rgb32 format (but not necessarily transparency!)
-        encodings.append("rgb32")
+        def add(x):
+            if x in self.allowed_encodings and x not in encodings:
+                encodings.append(x)
+        add("rgb32")
         #gtk2 can handle 'png' and 'jpeg' natively (without PIL)
         #(though using PIL is better since we can do that in the decode thread)
         for x in ("png", "jpeg"):
-            if x not in encodings:
-                encodings.append(x)
+            add(x)
         return encodings
 
     def has_transparency(self):
@@ -216,10 +238,20 @@ class XpraClient(GTKXpraClient):
     def setup_clipboard_helper(self, helperClass, *args, **kwargs):
         clipboardlog("setup_clipboard_helper(%s, %s, %s)", helperClass, args, kwargs)
         def clipboard_send(*parts):
-            if self.clipboard_enabled:
-                self.send(*parts)
-            else:
+            if not self.clipboard_enabled:
                 clipboardlog("clipboard is disabled, not sending clipboard packet")
+                return
+            #handle clipboard compression if needed:
+            packet = list(parts)
+            for i in range(len(packet)):
+                v = packet[i]
+                if type(v)==Uncompressed:
+                    #register the compressor which will fire in protocol.encode:
+                    def compress_clipboard():
+                        clipboardlog("compress_clipboard() compressing %s", args, v)
+                        return self.compressed_wrapper(v.datatype, v.data)
+                    v.compress = compress_clipboard
+            self.send(*packet)
         def clipboard_progress(local_requests, remote_requests):
             clipboardlog("clipboard_progress(%s, %s)", local_requests, remote_requests)
             if local_requests is not None:
@@ -258,6 +290,18 @@ class XpraClient(GTKXpraClient):
             self.tray.set_blinking(False)
 
 
+    def compressed_wrapper(self, datatype, data):
+        #FIXME: ugly assumptions here, should pass by name!
+        from xpra.net import compression
+        zlib = "zlib" in self.server_compressors and compression.use_zlib
+        lz4 = "lz4" in self.server_compressors and compression.use_lz4
+        lzo = "lzo" in self.server_compressors and compression.use_lzo
+        if zlib or lz4 or lzo:
+            return compression.compressed_wrapper(datatype, data, zlib=zlib, lz4=lz4, lzo=lzo, can_inline=False)
+        #we can't compress, so at least avoid warnings in the protocol layer:
+        return compression.Compressed("raw %s" % datatype, data, can_inline=True)
+
+
     def make_hello(self):
         capabilities = GTKXpraClient.make_hello(self)
         if xor_str is not None:
@@ -271,8 +315,8 @@ class XpraClient(GTKXpraClient):
         self._ui_packet_handlers["pointer-ungrab"] = self._process_pointer_ungrab
 
 
-    def process_ui_capabilities(self, capabilities):
-        GTKXpraClient.process_ui_capabilities(self, capabilities)
+    def process_ui_capabilities(self):
+        GTKXpraClient.process_ui_capabilities(self)
         global UI_watcher
         UI_watcher.start()
         #if server supports it, enable UI thread monitoring workaround when needed:
@@ -380,13 +424,16 @@ class XpraClient(GTKXpraClient):
         self.GLClientWindowClass = None
         self.opengl_props = {}
         from xpra.scripts.config import OpenGL_safety_check
-        check = OpenGL_safety_check()
-        if check:
+        use_gl, warning = OpenGL_safety_check()
+        if not use_gl:
+            log.warn("OpenGL disabled: %s", warning)
+            return
+        elif warning:
             if enable_opengl is True:
-                log.warn("OpenGL loading despite: %s", check)
+                log.warn("OpenGL loading despite: %s", warning)
             else:
-                self.opengl_props["info"] = "disabled: %s" % check
-                log.warn("OpenGL disabled: %s", check)
+                self.opengl_props["info"] = "disabled: %s" % warning
+                log.warn("OpenGL disabled: %s", warning)
                 return
         if enable_opengl is False:
             self.opengl_props["info"] = "disabled by configuration"
@@ -405,10 +452,12 @@ class XpraClient(GTKXpraClient):
             self.client_supports_opengl = True
             self.opengl_enabled = True
         except ImportError, e:
-            log.info("OpenGL support not enabled: %s", e)
+            log.warn("OpenGL support could not be enabled:")
+            log.warn(" %s", e)
             self.opengl_props["info"] = str(e)
         except Exception, e:
-            log.error("Error loading OpenGL support: %s", e, exc_info=True)
+            log.error("Error loading OpenGL support:")
+            log.error(" %s", e, exc_info=True)
             self.opengl_props["info"] = str(e)
 
     def get_group_leader(self, metadata, override_redirect):
