@@ -15,9 +15,10 @@ workspacelog = Logger("workspace")
 log = Logger("window")
 keylog = Logger("keyboard")
 
-from xpra.util import AdHocStruct, nn, bytestostr
+from xpra.util import AdHocStruct, bytestostr
 from xpra.gtk_common.gobject_compat import import_gtk, import_gdk, import_cairo, import_pixbufloader
 from xpra.gtk_common.gtk_util import pixbuf_new_from_data, COLORSPACE_RGB
+from xpra.gtk_common.keymap import KEY_TRANSLATIONS
 from xpra.client.client_window_base import ClientWindowBase
 gtk     = import_gtk()
 gdk     = import_gdk()
@@ -26,7 +27,7 @@ PixbufLoader = import_pixbufloader()
 
 CAN_SET_WORKSPACE = False
 HAS_X11_BINDINGS = False
-if os.name=="posix":
+if os.name=="posix" and os.environ.get("XPRA_SET_WORKSPACE", "1")!="0":
     try:
         from xpra.x11.gtk_x11.prop import prop_get, prop_set
         from xpra.x11.bindings.window_bindings import constants, X11WindowBindings  #@UnresolvedImport
@@ -55,6 +56,22 @@ except:
     unpremultiply_argb, byte_buffer_to_buffer  = None, None
 
 
+#window types we map to POPUP rather than TOPLEVEL
+POPUP_TYPE_HINTS = set((
+                    #"DIALOG",
+                    "MENU",
+                    #"TOOLBAR",
+                    "SPLASHSCREEN",
+                    #"UTILITY",
+                    "DOCK",
+                    #"DESKTOP",
+                    "DROPDOWN_MENU",
+                    "POPUP_MENU",
+                    "TOOLTIP",
+                    "NOTIFICATION",
+                    "COMBO",
+                    "DND"))
+
 
 class GTKKeyEvent(AdHocStruct):
     pass
@@ -72,6 +89,17 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
         ClientWindowBase.init_window(self, metadata)
         self._can_set_workspace = HAS_X11_BINDINGS and CAN_SET_WORKSPACE
 
+    def _is_popup(self, metadata):
+        #decide if the window type is POPUP or NORMAL:
+        if self._override_redirect:
+            return True
+        window_types = metadata.get("window-type", [])
+        popup_types = list(POPUP_TYPE_HINTS.intersection(window_types))
+        log("popup_types(%s)=%s", window_types, popup_types)
+        if popup_types:
+            log("forcing POPUP window type for %s", popup_types)
+            return True
+        return False
 
     def setup_window(self):
         self.set_app_paintable(True)
@@ -213,10 +241,11 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
             workspacelog("not sure if the window is shown or not: %s vs %s, resuming to be safe", desktop_workspace, window_workspace)
             suspend_resume = False
         elif desktop_workspace!=window_workspace:
-            workspacelog("window is on a different workspace, increasing its batch delay")
+            workspacelog("window is on a different workspace, increasing its batch delay (desktop: %s, window: %s)", desktop_workspace, window_workspace)
             suspend_resume = True
         elif self._window_workspace!=self._desktop_workspace:
-            workspacelog("window was on a different workspace, resetting its batch delay")
+            assert desktop_workspace==window_workspace
+            workspacelog("window was on a different workspace, resetting its batch delay (was desktop: %s, window: %s, now both on %s)", self._window_workspace, self._desktop_workspace, desktop_workspace)
             suspend_resume = False
         self._client.control_refresh(self._id, suspend_resume, refresh=False, options=options, client_properties=client_properties)
         self._window_workspace = window_workspace
@@ -442,13 +471,17 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
         return pointer, modifiers, buttons
 
     def parse_key_event(self, event, pressed):
+        keyval = event.keyval
+        keycode = event.hardware_keycode
+        keyname = gdk.keyval_name(keyval)
+        keyname = KEY_TRANSLATIONS.get((keyname, keyval, keycode), keyname)
         key_event = GTKKeyEvent()
         key_event.modifiers = self._client.mask_to_names(event.state)
-        key_event.keyname = nn(gdk.keyval_name(event.keyval))
-        key_event.keyval = nn(event.keyval)
-        key_event.keycode = event.hardware_keycode
+        key_event.keyname = keyname or ""
+        key_event.keyval = keyval or 0
+        key_event.keycode = keycode
         key_event.group = event.group
-        key_event.string = nn(event.string)
+        key_event.string = event.string or ""
         key_event.pressed = pressed
         keylog("parse_key_event(%s, %s)=%s", event, pressed, key_event)
         return key_event
