@@ -117,7 +117,7 @@ class SessionInfo(gtk.Window):
         tb.new_row("Xpra", label(__version__), label(self.client._remote_version or "unknown"))
         cl_rev, cl_ch, cl_date = "unknown", "", ""
         try:
-            from xpra.build_info import BUILD_DATE as cl_date
+            from xpra.build_info import BUILD_DATE as cl_date, BUILD_TIME as cl_time
             from xpra.src_info import REVISION as cl_rev, LOCAL_MODIFICATIONS as cl_ch      #@UnresolvedImport
         except:
             pass
@@ -137,13 +137,23 @@ class SessionInfo(gtk.Window):
             return None
         def server_version_info(*prop_names):
             return make_version_str(server_info(*prop_names))
-        tb.new_row("Revision", label(cl_rev), label(make_version_str(self.client._remote_revision)))
-        tb.new_row("Local Changes", label(cl_ch), label(server_version_info("build.local_modifications", "local_modifications")))
-        tb.new_row("Build date", label(cl_date), label(server_info("build_date", "build.date")))
+        def make_revision_str(rev, changes):
+            if not changes:
+                return rev
+            return "%s (%s changes)" % (rev, changes)
+        def make_datetime(date, time):
+            if not time:
+                return date
+            return "%s %s" % (date, time)
+        tb.new_row("Revision", label(make_revision_str(cl_rev, cl_ch)),
+                               label(make_revision_str(self.client._remote_revision, server_version_info("build.local_modifications", "local_modifications"))))
+        tb.new_row("Build date", label(make_datetime(cl_date, cl_time)),
+                                 label(make_datetime(server_info("build_date", "build.date"), server_info("build.time"))))
         gtk_version_info = get_gtk_version_info()
         def client_vinfo(prop, fallback="unknown"):
             k = "%s.version" % prop
-            return label(gtk_version_info.get(k, fallback))
+            return label(make_version_str(gtk_version_info.get(k, fallback)))
+        tb.new_row("Glib",      client_vinfo("glib"),       label(server_version_info("glib.version")))
         tb.new_row("Gobject",   client_vinfo("gobject"),    label(server_version_info("gobject.version", "pygtk_version")))
         tb.new_row("PyGTK",     client_vinfo("pygtk", ""),  label(server_version_info("pygtk.version", "pygtk_version")))
         tb.new_row("GTK",       client_vinfo("gtk"),        label(server_version_info("gtk.version", "gtk_version")))
@@ -629,22 +639,14 @@ class SessionInfo(gtk.Window):
         self.client_encodings_label.set_text(", ".join(sorted(self.client.get_core_encodings())))
 
         def get_encoder_list(caps):
-            l = []
-            if caps.get("bencode", True):
-                l.append("bencode")
-            if caps.get("rencode", False):
-                l.append("rencode")
-            return l
+            from xpra.net import packet_encoding
+            return [x for x in packet_encoding.ALL_ENCODERS if caps.get(x)]
         self.client_packet_encoders_label.set_text(", ".join(get_encoder_list(get_network_caps())))
         self.server_packet_encoders_label.set_text(", ".join(get_encoder_list(self.client.server_capabilities)))
 
         def get_compressor_list(caps):
-            l = []
-            if caps.get("zlib", False):
-                l.append("zlib")
-            if caps.get("lz4", False):
-                l.append("lz4")
-            return l
+            from xpra.net import compression
+            return [x for x in compression.ALL_COMPRESSORS if caps.get(x)]
         self.client_packet_compressors_label.set_text(", ".join(get_compressor_list(get_network_caps())))
         self.server_packet_compressors_label.set_text(", ".join(get_compressor_list(self.client.server_capabilities)))
         return False
@@ -690,13 +692,13 @@ class SessionInfo(gtk.Window):
         set_sound_info(self.microphone_label, None, self.client.microphone_enabled, self.client.sound_source)
 
         self.connection_type_label.set_text(c.info)
-        protocol_state = p.save_state()
-        level = protocol_state.get("compression_level")
-        if level==0:
-            compression_str = "None"
-        else:
-            compression_str = " + ".join([x for x in ("zlib", "lz4", "bencode", "rencode") if protocol_state.get(x, False)==True])
-            compression_str += ", level %s" % level
+        protocol_info = p.get_info()
+        encoder = protocol_info.get("encoder", "bug")
+        compressor = protocol_info.get("compressor", "none")
+        level = protocol_info.get("compression_level", 0)
+        compression_str = encoder + " + "+compressor
+        if level>0:
+            compression_str += " (level %s)" % level
         self.compression_label.set_text(compression_str)
 
         def enclabel(label, cipher):
