@@ -32,13 +32,13 @@ from xpra.platform import set_application_name
 from xpra.platform.features import MMAP_SUPPORTED, SYSTEM_TRAY_SUPPORTED, CLIPBOARD_WANT_TARGETS, CLIPBOARD_GREEDY, CLIPBOARDS
 from xpra.platform.gui import init as gui_init, ready as gui_ready, get_native_notifier_classes, get_native_tray_classes, get_native_system_tray_classes, get_native_tray_menu_helper_classes, ClientExtras
 from xpra.codecs.codec_constants import get_PIL_decodings
-from xpra.codecs.loader import codec_versions, has_codec, get_codec, PREFERED_ENCODING_ORDER, ALL_NEW_ENCODING_NAMES_TO_OLD, OLD_ENCODING_NAMES_TO_NEW
-from xpra.codecs.video_helper import getVideoHelper, ALL_VIDEO_DECODER_OPTIONS, NO_GFX_CSC_OPTIONS
+from xpra.codecs.loader import codec_versions, has_codec, get_codec, PREFERED_ENCODING_ORDER, ALL_NEW_ENCODING_NAMES_TO_OLD, OLD_ENCODING_NAMES_TO_NEW, PROBLEMATIC_ENCODINGS
+from xpra.codecs.video_helper import getVideoHelper, NO_GFX_CSC_OPTIONS
 from xpra.simple_stats import std_unit
-from xpra.net.protocol import Compressed, use_lz4
+from xpra.net import compression, packet_encoding
 from xpra.daemon_thread import make_daemon_thread
 from xpra.os_util import thread, Queue, os_info, platform_name, get_machine_id, get_user_uuid, bytestostr
-from xpra.util import nn, nonl, std, AtomicInteger, AdHocStruct, log_screen_sizes, typedict
+from xpra.util import nonl, std, AtomicInteger, AdHocStruct, log_screen_sizes, typedict, CLIENT_EXIT
 try:
     from xpra.clipboard.clipboard_base import ALL_CLIPBOARDS
 except:
@@ -100,6 +100,7 @@ class UIXpraClient(XpraClientBase):
         self.server_last_info = None
         self.info_request_pending = False
         self.screen_size_change_pending = False
+        self.allowed_encodings = []
         self.core_encodings = None
         self.encoding = None
 
@@ -165,6 +166,7 @@ class UIXpraClient(XpraClientBase):
         self.server_generic_encodings = False
         self.server_encodings = []
         self.server_core_encodings = []
+        self.server_encodings_problematic = PROBLEMATIC_ENCODINGS
         self.server_encodings_with_speed = ()
         self.server_encodings_with_quality = ()
         self.server_encodings_with_lossless = ()
@@ -207,12 +209,12 @@ class UIXpraClient(XpraClientBase):
         self._last_screen_settings = None
         self._suspended_at = 0
 
-        self.init_packet_handlers()
         self.init_aliases()
 
 
     def init(self, opts):
         """ initialize variables from configuration """
+        self.allowed_encodings = opts.encodings
         self.encoding = opts.encoding
         self.title = opts.title
         self.session_name = opts.session_name
@@ -251,11 +253,11 @@ class UIXpraClient(XpraClientBase):
         #until we add the ability to choose decoders, use all of them:
         #(and default to non grahics card csc modules if not specified)
         vh = getVideoHelper()
-        vh.set_modules(video_decoders=ALL_VIDEO_DECODER_OPTIONS, csc_modules=opts.csc_modules or NO_GFX_CSC_OPTIONS)
+        vh.set_modules(video_decoders=opts.video_decoders, csc_modules=opts.csc_modules or NO_GFX_CSC_OPTIONS)
         vh.init()
 
 
-    def init_ui(self, opts):
+    def init_ui(self, opts, extra_args=[]):
         """ initialize user interface """
         self.init_opengl(opts.opengl)
 
@@ -294,7 +296,7 @@ class UIXpraClient(XpraClientBase):
             self.client_extras = ClientExtras(self, opts)
 
         if opts.border:
-            self.parse_border(opts.border)
+            self.parse_border(opts.border, extra_args)
 
         #draw thread:
         self._draw_queue = Queue()
@@ -306,7 +308,7 @@ class UIXpraClient(XpraClientBase):
             self.init_mmap(self.mmap_group, conn.filename)
 
 
-    def parse_border(self, border_str):
+    def parse_border(self, border_str, extra_args):
         #not implemented here (see gtk2 client)
         pass
 
@@ -412,6 +414,9 @@ class UIXpraClient(XpraClientBase):
     def show_session_info(self, *args):
         log.warn("show_session_info() is not implemented in %s", self)
 
+    def show_bug_report(self, *args):
+        log.warn("show_bug_report() is not implemented in %s", self)
+
 
     def get_encodings(self):
         """
@@ -442,7 +447,7 @@ class UIXpraClient(XpraClientBase):
         core_encodings = ["rgb24"]
         #PIL:
         core_encodings += get_PIL_decodings(get_codec("PIL"))
-        if (has_codec("dec_webm") or has_codec("dec_webp")) and "webp" not in core_encodings:
+        if (has_codec("dec_webp")) and "webp" not in core_encodings:
             core_encodings.append("webp")
         #we enable all the video decoders we know about,
         #what will actually get used by the server will still depend on the csc modes supported
@@ -452,7 +457,7 @@ class UIXpraClient(XpraClientBase):
             if encoding not in core_encodings:
                 core_encodings.append(encoding)
         #remove duplicates and use prefered encoding order:
-        core_encodings = [x for x in PREFERED_ENCODING_ORDER if x in set(core_encodings)]
+        core_encodings = [x for x in PREFERED_ENCODING_ORDER if x in set(core_encodings) and x in self.allowed_encodings]
         log("do_get_core_encodings()=%s", core_encodings)
         return core_encodings
 
@@ -544,7 +549,7 @@ class UIXpraClient(XpraClientBase):
             traylog("xpra_tray_mouseover(%s)", args)
         def xpra_tray_exit(*args):
             traylog("xpra_tray_exit(%s)", args)
-            self.quit(0)
+            self.disconnect_and_quit(0, CLIENT_EXIT)
         def xpra_tray_geometry(*args):
             traylog("xpra_tray_geometry%s geometry=%s", args, tray.get_geometry())
         menu = None
@@ -660,9 +665,14 @@ class UIXpraClient(XpraClientBase):
         log("init_mmap(%s, %s)", mmap_group, socket_filename)
         from xpra.os_util import get_int_uuid
         from xpra.net.mmap_pipe import init_client_mmap
+        #calculate size:
+        root_w, root_h = self.get_root_size()
+        #at least 128MB, or 8 fullscreen RGBX frames:
+        mmap_size = max(128*1024*1024, root_w*root_h*4*8)
+        mmap_size = min(1024*1024*1024, mmap_size)
         self.mmap_token = get_int_uuid()
         self.mmap_enabled, self.mmap, self.mmap_size, self.mmap_tempfile, self.mmap_filename = \
-            init_client_mmap(self.mmap_token, mmap_group, socket_filename)
+            init_client_mmap(self.mmap_token, mmap_group, socket_filename, mmap_size)
 
     def clean_mmap(self):
         log("XpraClient.clean_mmap() mmap_filename=%s", self.mmap_filename)
@@ -677,14 +687,6 @@ class UIXpraClient(XpraClientBase):
         self.opengl_props = {"info" : "not supported"}
 
 
-    def send_layout(self):
-        keylog("send_layout()")
-        self.send("layout-changed", nn(self.keyboard_helper.xkbmap_layout), nn(self.keyboard_helper.xkbmap_variant))
-
-    def send_keymap(self):
-        keylog("send_keymap()")
-        self.send("keymap-changed", self.get_keymap_properties())
-
     def get_keymap_properties(self):
         props = self.keyboard_helper.get_keymap_properties()
         props["modifiers"] = self.get_current_modifiers()
@@ -820,16 +822,19 @@ class UIXpraClient(XpraClientBase):
         else:
             for k,v in self.get_keymap_properties().items():
                 capabilities[k] = v
-            capabilities["xkbmap_layout"] = nn(self.keyboard_helper.xkbmap_layout)
-            capabilities["xkbmap_variant"] = nn(self.keyboard_helper.xkbmap_variant)
+            capabilities["xkbmap_layout"] = self.keyboard_helper.xkbmap_layout or ""
+            capabilities["xkbmap_variant"] = self.keyboard_helper.xkbmap_variant or ""
             #show the user a summary of what we have detected:
             kb_info = {}
             xkbq = capabilities.get("xkbmap_query")
-            if xkbq:
-                from xpra.keyboard.layouts import parse_xkbmap_query
-                d = parse_xkbmap_query(xkbq)
+            xkbqs = capabilities.get("xkbmap_query_struct")
+            if xkbqs or xkbq:
+                if not xkbqs:
+                    #parse query into a dict
+                    from xpra.keyboard.layouts import parse_xkbmap_query
+                    xkbqs = parse_xkbmap_query(xkbq)
                 for x in ["rules", "model", "layout"]:
-                    v = d.get(x)
+                    v = xkbqs.get(x)
                     if v:
                         kb_info[x] = v
             if self.keyboard_helper.xkbmap_layout:
@@ -871,6 +876,7 @@ class UIXpraClient(XpraClientBase):
             except:
                 pass
         capabilities.update({
+            "wants_events"              : True,
             "randr_notify"              : True,
             "compressible_cursors"      : True,
             "dpi"                       : self.dpi,
@@ -895,7 +901,6 @@ class UIXpraClient(XpraClientBase):
             "encoding.video_scaling"    : True,
             #separate plane is only supported by avcodec2:
             "encoding.video_separateplane"  : get_codec("dec_avcodec") is None and get_codec("dec_avcodec2") is not None,
-            "encoding.rgb_lz4"          : use_lz4 and self.compression_level==1,
             "encoding.webp_leaks"       : False,
             "encoding.transparency"     : self.has_transparency(),
             "rgb24zlib"                 : True,
@@ -917,14 +922,15 @@ class UIXpraClient(XpraClientBase):
             "encodings"                 : self.get_encodings(),
             "encodings.core"            : self.get_core_encodings(),
             })
+        #generic rgb compression flags:
+        for x in compression.ALL_COMPRESSORS:
+            capabilities["encoding.rgb_%s" % x] = x in compression.get_enabled_compressors()
+
         control_commands = ["show_session_info", "enable_bencode", "enable_zlib"]
-        from xpra.net.protocol import use_bencode, use_rencode
-        if use_lz4:
-            control_commands.append("enable_lz4")
-        if use_bencode:
-            control_commands.append("enable_bencode")
-        if use_rencode:
-            control_commands.append("enable_rencode")
+        for x in compression.get_enabled_compressors():
+            control_commands.append("enable_"+x)
+        for x in packet_encoding.get_enabled_encoders():
+            control_commands.append("enable_"+x)
         capabilities["control_commands"] = control_commands
         for k,v in codec_versions.items():
             capabilities["encoding.%s.version" % k] = v
@@ -992,21 +998,32 @@ class UIXpraClient(XpraClientBase):
         if iq<0:
             iq = 70
         capabilities["encoding.initial_quality"] = iq
+        sound_caps = {}
         try:
-            from xpra.sound.gstreamer_util import has_gst, add_gst_capabilities
-        except:
-            has_gst = False
-        if has_gst:
+            import xpra.sound
+            log("loaded %s", xpra.sound)
             try:
-                from xpra.sound.pulseaudio_util import add_pulseaudio_capabilities
-                add_pulseaudio_capabilities(capabilities)
-                add_gst_capabilities(capabilities, receive=self.speaker_allowed, send=self.microphone_allowed,
-                                     receive_codecs=self.speaker_codecs, send_codecs=self.microphone_codecs, new_namespace=True)
-                soundlog("sound capabilities: %s", [(k,v) for k,v in capabilities.items() if k.startswith("sound.")])
+                from xpra.sound.gstreamer_util import has_gst, get_info as get_gst_info
+                sound_caps.update(get_gst_info(receive=self.speaker_allowed, send=self.microphone_allowed,
+                                     receive_codecs=self.speaker_codecs, send_codecs=self.microphone_codecs))
             except Exception, e:
                 log.error("failed to setup sound: %s", e, exc_info=True)
                 self.speaker_allowed = False
                 self.microphone_allowed = False
+                has_gst = False
+            if has_gst:
+                try:
+                    from xpra.sound.pulseaudio_util import get_info as get_pa_info
+                    sound_caps.update(get_pa_info())
+                    sound_caps.update(get_gst_info(receive=self.speaker_allowed, send=self.microphone_allowed,
+                                         receive_codecs=self.speaker_codecs, send_codecs=self.microphone_codecs))
+                except Exception, e:
+                    pass
+            from xpra.util import updict
+            updict(capabilities, "sound", sound_caps)
+            soundlog("sound capabilities: %s", sound_caps)
+        except ImportError, e:
+            soundlog.warn("sound support not available: %s", e)
         #batch options:
         for bprop in ("always", "min_delay", "max_delay", "delay", "max_events", "max_pixels", "time_unit"):
             evalue = os.environ.get("XPRA_BATCH_%s" % bprop.upper())
@@ -1053,6 +1070,7 @@ class UIXpraClient(XpraClientBase):
     def check_echo_timeout(self, ping_time):
         log("check_echo_timeout(%s) last_ping_echoed_time=%s", ping_time, self.last_ping_echoed_time)
         if self.last_ping_echoed_time<ping_time:
+            #no point trying to use disconnect_and_quit() to tell the server here..
             self.warn_and_quit(EXIT_TIMEOUT, "server ping timeout - waited %s seconds without a response" % PING_TIMEOUT)
 
     def send_ping(self):
@@ -1094,6 +1112,10 @@ class UIXpraClient(XpraClientBase):
         self.send("ping_echo", echotime, l1, l2, l3, int(1000.0*sl))
 
 
+    def _process_server_event(self, packet):
+        log(": ".join(packet[1:]))
+
+
     def _process_info_response(self, packet):
         self.info_request_pending = False
         self.server_last_info = packet[1]
@@ -1132,9 +1154,16 @@ class UIXpraClient(XpraClientBase):
         self.send("min-speed", s)
 
 
-    def parse_server_capabilities(self, c):
-        if not XpraClientBase.parse_server_capabilities(self, c):
-            return
+    def server_connection_established(self):
+        if XpraClientBase.server_connection_established(self):
+            #process the rest from the UI thread:
+            self.idle_add(self.process_ui_capabilities)
+
+
+    def parse_server_capabilities(self):
+        if not XpraClientBase.parse_server_capabilities(self):
+            return  False
+        c = self.server_capabilities
         if not self.session_name:
             self.session_name = c.strget("session_name", "")
         set_application_name(self.session_name or "Xpra")
@@ -1150,6 +1179,7 @@ class UIXpraClient(XpraClientBase):
         self.bell_enabled = self.server_supports_bell and self.client_supports_bell
         self.server_supports_clipboard = c.boolget("clipboard")
         self.server_clipboards = c.strlistget("clipboards", ALL_CLIPBOARDS)
+        self.server_compressors = c.strlistget("compressors", ["zlib"])
         self.clipboard_enabled = self.client_supports_clipboard and self.server_supports_clipboard
         self.server_dbus_proxy = c.boolget("dbus_proxy")
         self.mmap_enabled = self.supports_mmap and self.mmap_enabled and c.boolget("mmap_enabled")
@@ -1176,6 +1206,7 @@ class UIXpraClient(XpraClientBase):
         self.server_generic_encodings = c.boolget("encoding.generic")
         self.server_encodings = getenclist("encodings")
         self.server_core_encodings = getenclist("encodings.core", self.server_encodings)
+        self.server_encodings_problematic = getenclist("encodings.problematic", PROBLEMATIC_ENCODINGS)  #server is telling us to try to avoid those
         self.server_encodings_with_speed = getenclist("encodings.with_speed", ("h264",)) #old servers only supported x264
         self.server_encodings_with_quality = getenclist("encodings.with_quality", ("jpeg", "webp", "h264"))
         self.server_encodings_with_lossless_mode = getenclist("encodings.with_lossless_mode", ())
@@ -1224,10 +1255,9 @@ class UIXpraClient(XpraClientBase):
             if proxy_hostname:
                 msg += " on '%s'" % std(proxy_hostname)
             log.info(msg)
-        #process the rest from the UI thread:
-        self.idle_add(self.process_ui_capabilities, c)
+        return True
 
-    def process_ui_capabilities(self, c):
+    def process_ui_capabilities(self):
         #figure out the maximum actual desktop size and use it to
         #calculate the maximum size of a packet (a full screen update packet)
         if self.clipboard_enabled:
@@ -1235,6 +1265,7 @@ class UIXpraClient(XpraClientBase):
             self.clipboard_enabled = self.clipboard_helper is not None
         self.set_max_packet_size()
         self.send_deflate_level()
+        c = self.server_capabilities
         server_desktop_size = c.intlistget("desktop_size")
         log("server desktop size=%s", server_desktop_size)
         if not c.boolget("shadow"):
@@ -1279,6 +1310,10 @@ class UIXpraClient(XpraClientBase):
         if self.toggle_keyboard_sync:
             self.connect("keyboard-sync-toggled", self.send_keyboard_sync_enabled_status)
         self.send_ping()
+        if self.pings:
+            self.timeout_add(1000, self.send_ping)
+        else:
+            self.timeout_add(10*1000, self.send_ping)
         if not c.boolget("notify-startup-complete"):
             #we won't get notified, so assume it is now:
             self._startup_complete()
@@ -1344,18 +1379,14 @@ class UIXpraClient(XpraClientBase):
             args = packet[2:]
             log("calling show_session_info%s on server request", args)
             self.show_session_info(*args)
-        elif command=="enable_zlib":
-            log.info("switching to zlib on server request")
-            self._protocol.enable_zlib()
-        elif command=="enable_lz4":
-            log.info("switching to lz4 on server request")
-            self._protocol.enable_lz4()
-        elif command=="enable_bencode":
-            log.info("switching to bencode on server request")
+        elif command in ("enable_%x" for x in compression.get_enabled_compressors()):
+            compressor = command.split("_")[1]
+            log.info("switching to %s on server request", compressor)
+            self._protocol.enable_compressor(compressor)
+        elif command in ("enable_%x" for x in packet_encoding.get_enabled_encoders()):
+            packet_encoding = command.split("_")[1]
+            log.info("switching to %s on server request", packet_encoding)
             self._protocol.enable_bencode()
-        elif command=="enable_rencode":
-            log.info("switching to rencode on server request")
-            self._protocol.enable_rencode()
         elif command=="name":
             assert len(args)>=3
             self.session_name = args[2]
@@ -1541,7 +1572,7 @@ class UIXpraClient(XpraClientBase):
     def new_sound_buffer(self, sound_source, data, metadata):
         soundlog("new_sound_buffer(%s, %s, %s) sound source=%s", sound_source, len(data or []), metadata, self.sound_source)
         if self.sound_source:
-            self.send("sound-data", self.sound_source.codec, Compressed(self.sound_source.codec, data), metadata)
+            self.send("sound-data", self.sound_source.codec, compression.Compressed(self.sound_source.codec, data), metadata)
 
     def _process_sound_data(self, packet):
         codec, data, metadata = packet[1:4]
@@ -1915,10 +1946,16 @@ class UIXpraClient(XpraClientBase):
         log("set maximum packet size to %s", self._protocol.max_packet_size)
 
 
-    def init_packet_handlers(self):
-        XpraClientBase.init_packet_handlers(self)
+    def init_authenticated_packet_handlers(self):
+        log("init_authenticated_packet_handlers()")
+        XpraClientBase.init_authenticated_packet_handlers(self)
+        def delhandler(k):
+            #remove any existing mapping:
+            if k in self._packet_handlers:
+                del self._packet_handlers[k]
+            if k in self._ui_packet_handlers:
+                del self._ui_packet_handlers[k]
         for k,v in {
-            "hello":                self._process_hello,
             "startup-complete":     self._startup_complete,
             "new-window":           self._process_new_window,
             "new-override-redirect":self._process_new_override_redirect,
@@ -1941,6 +1978,7 @@ class UIXpraClient(XpraClientBase):
             "draw":                 self._process_draw,
             # "clipboard-*" packets are handled by a special case below.
             }.items():
+            delhandler(k)
             self._ui_packet_handlers[k] = v
         #these handlers can run directly from the network thread:
         for k,v in {
@@ -1948,7 +1986,9 @@ class UIXpraClient(XpraClientBase):
             "ping_echo":            self._process_ping_echo,
             "info-response":        self._process_info_response,
             "sound-data":           self._process_sound_data,
+            "server-event":         self._process_server_event,
             }.items():
+            delhandler(k)
             self._packet_handlers[k] = v
 
     def process_clipboard_packet(self, packet):
