@@ -4,14 +4,12 @@
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import zlib
-
 from xpra.log import Logger
 log = Logger("paint")
 
 from threading import Lock
 from xpra.net.mmap_pipe import mmap_read
-from xpra.net.protocol import has_lz4, LZ4_uncompress
+from xpra.net import compression
 from xpra.util import typedict
 from xpra.codecs.codec_constants import get_colorspace_from_avutil_enum, get_PIL_decodings
 from xpra.codecs.loader import get_codec
@@ -180,11 +178,11 @@ class WindowBackingBase(object):
         """
         img_data = raw_data
         if options:
-            if options.intget("zlib", 0)>0:
-                img_data = zlib.decompress(raw_data)
-            elif options.boolget("lz4", False):
-                assert has_lz4
-                img_data = LZ4_uncompress(raw_data)
+            #check for one of the compressors:
+            comp = [x for x in compression.ALL_COMPRESSORS if options.intget(x, 0)]
+            if comp:
+                assert len(comp)==1, "more than one compressor specified: %s" % str(comp)
+                img_data = compression.decompress_by_name(raw_data, algo=comp[0])
         if len(img_data)!=rowstride * height:
             log.error("invalid img data %s: %s", type(img_data), str(img_data)[:256])
             raise Exception("expected %s bytes for %sx%s with rowstride=%s but received %s (%s compressed)" %
@@ -255,12 +253,6 @@ class WindowBackingBase(object):
         return False
 
     def paint_webp(self, img_data, x, y, width, height, options, callbacks):
-        dec_webp = get_codec("dec_webp")
-        if dec_webp:
-            return self.paint_webp_using_cwebp(img_data, x, y, width, height, options, callbacks)
-        return self.paint_webp_using_webm(img_data, x, y, width, height, options, callbacks)
-
-    def paint_webp_using_cwebp(self, img_data, x, y, width, height, options, callbacks):
         dec_webp = get_codec("dec_webp")
         has_alpha = options.get("has_alpha", False)
         buffer_wrapper, width, height, stride, has_alpha, rgb_format = dec_webp.decompress(img_data, has_alpha)
@@ -274,28 +266,6 @@ class WindowBackingBase(object):
         else:
             return self.paint_rgb24(data, x, y, width, height, stride, options, callbacks)
 
-    def paint_webp_using_webm(self, img_data, x, y, width, height, options, callbacks):
-        """ can be called from any thread """
-        dec_webm = get_codec("dec_webm")
-        assert dec_webm is not None, "webp decoder not found"
-        paint_options = typedict(options)
-        if options.get("has_alpha", False):
-            decode = dec_webm.DecodeRGBA
-            rowstride = width*4
-            paint_rgb = self.do_paint_rgb32
-            paint_options["rgb_format"] = "RGBA"
-        else:
-            decode = dec_webm.DecodeRGB
-            rowstride = width*3
-            paint_rgb = self.do_paint_rgb24
-            paint_options["rgb_format"] = "RGB"
-        log("paint_webp(%s) using decode=%s, paint=%s, paint_options=%s",
-             ("%s bytes" % len(img_data), x, y, width, height, options, callbacks), decode, paint_rgb, paint_options)
-        rgb_data = decode(img_data)
-        pixels = str(rgb_data.bitmap)
-        self.idle_add(paint_rgb, pixels, x, y, width, height, rowstride, paint_options, callbacks)
-        return  False
-
     def paint_rgb24(self, raw_data, x, y, width, height, rowstride, options, callbacks):
         """ called from non-UI thread
             this method calls process_delta before calling do_paint_rgb24 from the UI thread via idle_add
@@ -421,8 +391,10 @@ class WindowBackingBase(object):
 
             img = self._video_decoder.decompress_image(img_data, options)
             if not img:
-                raise Exception("paint_with_video_decoder: wid=%s, %s decompression error on %s bytes of picture data for %sx%s pixels using %s, options=%s" % (
-                      self.wid, coding, len(img_data), width, height, self._video_decoder, options))
+                fire_paint_callbacks(callbacks, False)
+                log.error("paint_with_video_decoder: wid=%s, %s decompression error on %s bytes of picture data for %sx%s pixels using %s, options=%s",
+                      self.wid, coding, len(img_data), width, height, self._video_decoder, options)
+                return False
             self.do_video_paint(img, x, y, enc_width, enc_height, width, height, options, callbacks)
         finally:
             self._decoder_lock.release()
