@@ -4,10 +4,14 @@
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+import sys
 import os.path
 from xpra.log import Logger
 log = Logger("codec", "loader")
 
+if sys.version > '3':
+    unicode = str           #@ReservedAssignment
+
 #these codecs may well not load because we
 #do not require the libraries to be installed
 NOWARN = ["nvenc", "opencl"]
@@ -22,6 +26,7 @@ def codec_import_check(name, description, top_module, class_module, *classnames)
             __import__(top_module, {}, {}, [])
         except ImportError, e:
             log(" cannot import %s (%s): %s", name, description, e)
+            log("", exc_info=True)
             codec_errors[name] = e
             return None
         #module is present
@@ -39,9 +44,11 @@ def codec_import_check(name, description, top_module, class_module, *classnames)
             if name in NOWARN:
                 l = log.debug
             l(" cannot import %s (%s): %s", name, description, e)
+            log("", exc_info=True)
     except Exception, e:
         codec_errors[name] = e
         log.warn(" cannot load %s (%s): %s missing from %s: %s", name, description, classname, class_module, e)
+        log("", exc_info=True)
     return None
 codec_versions = {}
 def add_codec_version(name, top_module, version="get_version()", alt_version=None):
@@ -70,8 +77,10 @@ def add_codec_version(name, top_module, version="get_version()", alt_version=Non
     except ImportError, e:
         #not present
         log(" cannot import %s: %s", name, e)
+        log("", exc_info=True)
     except Exception, e:
         log.warn("error during codec import: %s", e)
+        log("", exc_info=True)
     return None
 
 
@@ -122,23 +131,6 @@ def load_codecs():
     codec_import_check("dec_webp", "webp decoder", "xpra.codecs.webp", "xpra.codecs.webp.decode", "decompress")
     add_codec_version("dec_webp", "xpra.codecs.webp.decode")
 
-    #no bytearray (python 2.6 or later) or no bitmap handlers, no webm:
-    from xpra.os_util import builtins
-    webm_handlers = codec_import_check("webm_bitmap_handlers", "webp bitmap handler", "xpra.codecs.webm", "xpra.codecs.webm.handlers", "BitmapHandler")
-    if ("bytearray" in builtins.__dict__) and webm_handlers:
-        codec_import_check("enc_webm", "webp encoder", "xpra.codecs.webm", "xpra.codecs.webm.encode", \
-                           "EncodeRGB", "EncodeRGBA", "EncodeBGR", "EncodeBGRA", \
-                           "EncodeLosslessRGB", "EncodeLosslessRGBA", "EncodeLosslessBGRA", "EncodeLosslessBGR")
-
-        v = add_codec_version("webm", "xpra.codecs.webm", "__VERSION__")
-        MIN_V = "0.2.3"
-        if v<MIN_V:
-            log.warn("python-webm error: found version %s but the minimum required is %s", v, MIN_V)
-            log.warn(" webm decoding has been disabled to prevent memory leaks")
-        else:
-            #these symbols are all available upstream as of libwebp 0.2:
-            codec_import_check("dec_webm", "webp encoder", "xpra.codecs.webm", "xpra.codecs.webm.decode", "DecodeRGB", "DecodeRGBA", "DecodeBGR", "DecodeBGRA")
-
     log("done loading codecs")
     log("found:")
     #print("codec_status=%s" % codecs)
@@ -172,8 +164,6 @@ ALL_NEW_ENCODING_NAMES_TO_OLD = {"h264" : "x264", "vp8" : "vpx", "rgb" : "rgb24"
 ALL_CODECS = "PIL", "enc_vpx", "dec_vpx", "enc_x264", "enc_x265", "nvenc", \
             "csc_swscale", "csc_cython", "csc_opencl", \
             "dec_avcodec", "dec_avcodec2", \
-            "enc_webm", \
-            "dec_webm", \
             "enc_webp", \
             "dec_webp"
 
@@ -181,13 +171,10 @@ ALL_CODECS = "PIL", "enc_vpx", "dec_vpx", "enc_x264", "enc_x265", "nvenc", \
 #so we have both core encodings (rgb24/rgb32) and regular encodings (rgb) in here:
 PREFERED_ENCODING_ORDER = ["h264", "vp8", "png", "png/P", "png/L", "webp", "rgb", "rgb24", "rgb32", "jpeg", "h265", "vp9"]
 
-compressors = ["zlib"]
-try:
-    import lz4      #@UnresolvedImport
-    del lz4
-    compressors.append("lz4")
-except:
-    pass
+from xpra.net import compression
+RGB_COMP_OPTIONS  = ["Raw RGB"]
+if compression.get_enabled_compressors():
+    RGB_COMP_OPTIONS  += ["/".join(compression.get_enabled_compressors())]
 
 ENCODINGS_TO_NAME = {
       "h264"    : "H.264",
@@ -199,12 +186,12 @@ ENCODINGS_TO_NAME = {
       "png/L"   : "PNG (8bpp grayscale)",
       "webp"    : "WebP",
       "jpeg"    : "JPEG",
-      "rgb"     : "Raw RGB + %s (24/32bpp)" % ("/".join(compressors)),
+      "rgb"     : " + ".join(RGB_COMP_OPTIONS) + " (24/32bpp)",
     }
 
 ENCODINGS_HELP = {
       "h264"    : "H.264 video codec",
-      "h265"    : "H.265 (HEVC) video codec (slow)",
+      "h265"    : "H.265 (HEVC) video codec (slow and buggy - do not use!)",
       "vp8"     : "VP8 video codec",
       "vp9"     : "VP9 video codec (very slow - do not use!)",
       "png"     : "Portable Network Graphics (lossless, 24bpp or 32bpp for transparency)",
@@ -212,11 +199,15 @@ ENCODINGS_HELP = {
       "png/L"   : "Portable Network Graphics (lossy, 8bpp grayscale)",
       "webp"    : "WebP compression (lossless or lossy)",
       "jpeg"    : "JPEG lossy compression",
-      "rgb"     : "Raw RGB pixels, lossless, compressed using %s (24bpp or 32bpp for transparency)" % (" or ".join(compressors)),
+      "rgb"     : "Raw RGB pixels, lossless, compressed using %s (24bpp or 32bpp for transparency)" % (" or ".join(compression.get_enabled_compressors())),
       }
 
 HELP_ORDER = ("h264", "h265", "vp8", "vp9", "png", "png/P", "png/L", "webp", "rgb", "jpeg")
 
+#those are currently so useless that we don't want the user to select them by mistake
+PROBLEMATIC_ENCODINGS = ("h265", "vp9")
+
+
 def encodings_help(encodings):
     h = []
     for e in HELP_ORDER:
@@ -234,8 +225,8 @@ def main():
     from xpra.platform import init, clean
     try:
         init("Loader", "Encoding Info")
-        import sys
-        if "-v" in sys.argv or "--verbose" in sys.argv:
+        verbose = "-v" in sys.argv or "--verbose" in sys.argv
+        if verbose:
             log.enable_debug()
 
         load_codecs()
@@ -243,19 +234,40 @@ def main():
         #print("codec_status=%s" % codecs)
         for name in ALL_CODECS:
             mod = codecs.get(name, "")
+            f = mod
             if mod and hasattr(mod, "__file__"):
-                mod = mod.__file__
-                if mod.startswith(os.getcwd()):
-                    mod = mod[len(os.getcwd()):]
-                    if mod.startswith(os.path.sep):
-                        mod = mod[1:]
-            print("* %s : %s" % (name.ljust(20), mod))
+                f = mod.__file__
+                if f.startswith(os.getcwd()):
+                    f = f[len(os.getcwd()):]
+                    if f.startswith(os.path.sep):
+                        f = f[1:]
+            print("* %s : %s" % (name.ljust(20), f))
+            if mod and verbose:
+                try:
+                    if name=="PIL":
+                        #special case for PIL which can be used for both encoding and decoding:
+                        from xpra.codecs.codec_constants import get_PIL_encodings, get_PIL_decodings
+                        e = get_PIL_encodings(mod)
+                        print("                         ENCODE: %s" % ", ".join(e))
+                        d = get_PIL_decodings(mod)
+                        print("                         DECODE: %s" % ", ".join(d))
+                    elif name.find("enc")>=0 or name.find("dec")>=0:
+                        encodings = mod.get_encodings()
+                        print("                         %s" % ", ".join(encodings))
+                    elif name.find("csc")>=0:
+                        cs = list(mod.get_input_colorspaces())
+                        for c in list(cs):
+                            cs += list(mod.get_output_colorspaces(c))
+                        print("                         %s" % ", ".join(list(set(cs))))
+                except:
+                    e = sys.exc_info()[1]
+                    print("error getting extra information on %s: %s" % (name, e))
         print("")
         print("codecs versions:")
         def pver(v):
             if type(v)==tuple:
                 return ".".join([str(x) for x in v])
-            elif type(v)==str and v.startswith("v"):
+            elif type(v) in (str, unicode) and v.startswith("v"):
                 return v[1:]
             return str(v)
         for name in sorted(codec_versions.keys()):
