@@ -194,3 +194,12 @@ def parse_xkbmap_query(xkbmap_query):
             if v!=",":
                 settings[m.group(1)] = v
     return settings
+
+def xkbmap_query_tostring(query_dict):
+    """ converts an xkb query dict back into a string """
+    s = ""
+    for k in ("rules", "model", "layout", "variant"):
+        if k in query_dict:
+            v = query_dict.get(k)
+            s += (str(k)+":").ljust(12)+str(v)+"\n"
+    return s
