@@ -53,6 +53,7 @@ class Connection(object):
         self.output_bytecount = 0
         self.filename = None            #only used for unix domain sockets!
         self.active = True
+        self.timeout = 0
 
     def is_active(self):
         return self.active
@@ -77,7 +78,12 @@ class Connection(object):
         return r
 
     def get_info(self):
-        return self.info or ""
+        return {
+                "type"              : self.info or "",
+                "endpoint"          : self.target or "",
+                "input.bytecount"   : self.input_bytecount,
+                "output.bytecount"  : self.output_bytecount,
+                }
 
 
 # A simple, portable abstraction for a blocking, low-level
