@@ -17,7 +17,23 @@ def valid_dir(path):
 
 
 def get_default_conf_dir():
-    return os.environ.get("XPRA_CONF_DIR", "~/.xpra")
+    #some platforms may also ship a default config with the application
+    return None
+
+def get_system_conf_dir():
+    #the system wide configuration directory
+    if sys.prefix == '/usr':
+        #default posix config location:
+        default_conf_dir = '/etc/xpra'
+    else:
+        #hope the prefix is something like "/usr/local":
+        default_conf_dir = sys.prefix + '/etc/xpra/'
+    return os.environ.get("XPRA_SYSCONF_DIR", default_conf_dir)
+
+def get_user_conf_dir():
+    #per-user configuration location:
+    return os.environ.get("XPRA_USER_CONF_DIR", "~/.xpra")
+
 
 def get_default_socket_dir():
     return os.environ.get("XPRA_SOCKET_DIR", "~/.xpra")
@@ -109,4 +125,39 @@ platform_import(globals(), "paths", True,
                 "get_icon_dir")
 platform_import(globals(), "paths", False,
                 "get_default_conf_dir",
+                "get_system_conf_dir",
+                "get_user_conf_dir",
                 "get_default_socket_dir")
+
+def get_info():
+    return {
+            "default_conf.dir"  : get_default_conf_dir(),
+            "system_conf_dir"   : get_system_conf_dir(),
+            "user_conf_dir"     : get_user_conf_dir(),
+            "socket"            : get_default_socket_dir(),
+            "app"               : get_app_dir(),
+            "app.default"       : default_get_app_dir(),
+            "resources"         : get_resources_dir(),
+            "icons"             : get_icon_dir()
+            }
+
+
+def main():
+    if "-v" in sys.argv or "--verbose" in sys.argv:
+        from xpra.log import add_debug_category
+        add_debug_category("util")
+
+    from xpra.util import nonl, pver
+    def print_dict(d):
+        for k in sorted(d.keys()):
+            v = d[k]
+            print("* %s : %s" % (k.ljust(32), nonl(pver(v))))
+    from xpra.platform import init, clean
+    try:
+        init("Path-Info", "Path Info")
+        print_dict(get_info())
+    finally:
+        clean()
+
+if __name__ == "__main__":
+    main()
