@@ -10,11 +10,12 @@ import os
 import stat
 import socket
 import time
-from optparse import OptionParser, OptionGroup
+import optparse
 import logging
 from subprocess import Popen, PIPE
 import signal
 import shlex
+import traceback
 
 from xpra import __version__ as XPRA_VERSION
 from xpra.dotxpra import DotXpra, osexpand
@@ -39,6 +40,10 @@ def enabled_str(v):
 
 
 def warn(msg):
+    #use this function to print warnings
+    #we must write to stderr to prevent
+    #the output from interfering when running as proxy over ssh
+    #(which uses stdin / stdout as communication channel)
     sys.stderr.write(msg+"\n")
 
 def nox():
@@ -59,24 +64,60 @@ if supports_server:
         supports_server = False
 
 
+class InitException(Exception):
+    pass
+class InitInfo(Exception):
+    pass
+class InitExit(Exception):
+    def __init__(self, status, msg):
+        self.status = status
+        Exception.__init__(self, msg)
+
+#this parse doesn't exit when it encounters an error,
+#allowing us to deal with it better and show a UI message if needed.
+class ModifiedOptionParser(optparse.OptionParser):
+    def error(self, msg):
+        raise InitException(msg)
+    def exit(self, status=0, msg=None):
+        raise InitExit(status, msg)
+
+
 def main(script_file, cmdline):
-    from xpra.platform import init as platform_init, clean as platform_clean
+    from xpra.platform import init as platform_init, clean as platform_clean, command_error, command_info, get_main_fallback
+    if len(cmdline)==1:
+        fm = get_main_fallback()
+        if fm:
+            return fm()
+
     try:
         platform_init("Xpra")
         try:
             defaults = make_defaults_struct()
-            parser, options, args = do_parse_cmdline(cmdline, defaults)
+            options, args = do_parse_cmdline(cmdline, defaults)
             if not args:
                 print("xpra: need a mode")
                 return -1
             mode = args.pop(0)
-            return run_mode(script_file, parser, options, args, mode, defaults)
+            def err(*args):
+                raise InitException(*args)
+            return run_mode(script_file, err, options, args, mode, defaults)
         except SystemExit:
             raise
-        except:
+        except InitExit:
             e = sys.exc_info()[1]
-            print("xpra main exception: %s" % e)
-            sys.exit(1)
+            if str(e) and e.args and (e.args[0] or len(e.args)>1):
+                command_info("%s" % e)
+            return e.status
+        except InitInfo:
+            command_info("%s" % sys.exc_info()[1])
+            return 0
+        except InitException:
+            e = sys.exc_info()[1]
+            command_error("xpra initialization error: %s" % e)
+            return 1
+        except Exception:
+            command_error("xpra main error:\n%s" % traceback.format_exc())
+            return 1
     finally:
         platform_clean()
 
@@ -93,6 +134,93 @@ def fixup_debug_option(value):
     #if we're here, the value should be a CSV list of categories
     return value
 
+def _csvstr(value):
+    if type(value) in (tuple, list):
+        return ",".join(str(x).lower().strip() for x in value)
+    elif type(value)==str:
+        return value.strip().lower()
+    raise Exception("don't know how to convert %s to a csv list!" % type(value))
+
+def _nodupes(s):
+    from xpra.util import remove_dupes
+    return remove_dupes(x.strip().lower() for x in s.split(","))
+
+def fixup_video_all_or_none(options):
+    from xpra.codecs.video_helper import ALL_VIDEO_ENCODER_OPTIONS as aveco
+    from xpra.codecs.video_helper import ALL_CSC_MODULE_OPTIONS as acsco
+    from xpra.codecs.video_helper import ALL_VIDEO_DECODER_OPTIONS as avedo
+    vestr   = _csvstr(options.video_encoders)
+    cscstr  = _csvstr(options.csc_modules)
+    vdstr   = _csvstr(options.video_decoders)
+    def getlist(strarg, help_txt, all_list):
+        if strarg=="help":
+            raise InitInfo("the following %s may be available: %s" % (help_txt, ", ".join(all_list)))
+        elif strarg=="none":
+            return []
+        elif strarg=="all":
+            return all_list
+        else:
+            return _nodupes(strarg)
+    options.video_encoders  = getlist(vestr,    "video encoders",   aveco)
+    options.csc_modules     = getlist(cscstr,   "csc modules",      acsco)
+    options.video_decoders  = getlist(vdstr,    "video decoders",   avedo)
+
+def fixup_encodings(options):
+    from xpra.codecs.loader import ALL_OLD_ENCODING_NAMES_TO_NEW, PREFERED_ENCODING_ORDER
+    if options.encoding:
+        #fix old encoding names if needed:
+        options.encoding = ALL_OLD_ENCODING_NAMES_TO_NEW.get(options.encoding, options.encoding)
+    estr = _csvstr(options.encodings)
+    if estr=="all":
+        #replace with an actual list
+        options.encodings = PREFERED_ENCODING_ORDER
+        return
+    encodings = _nodupes(estr)
+    if "rgb" in encodings:
+        if "rgb24" not in encodings:
+            encodings.append("rgb24")
+        if "rgb32" not in encodings:
+            encodings.append("rgb32")
+    options.encodings = encodings
+
+def fixup_compression(options):
+    #packet compression:
+    from xpra.net import compression
+    cstr = _csvstr(options.compressors)
+    if cstr=="none":
+        compressors = []
+    elif cstr=="all":
+        compressors = compression.get_enabled_compressors()
+    else:
+        compressors = _nodupes(cstr)
+        unknown = [x for x in compressors if x and x not in compression.ALL_COMPRESSORS]
+        if unknown:
+            warn("warning: invalid compressor(s) specified: %s" % (", ".join(unknown)))
+    for c in compression.ALL_COMPRESSORS:
+        enabled = c in compression.get_enabled_compressors() and c in compressors
+        setattr(compression, "use_%s" % c, enabled)
+    if not compression.get_enabled_compressors():
+        #force compression level to zero since we have no compressors available:
+        options.compression_level = 0
+
+def fixup_packetencoding(options):
+    #packet encoding
+    from xpra.net import packet_encoding
+    pestr = _csvstr(options.packet_encoders)
+    if pestr=="all":
+        packet_encoders = packet_encoding.get_enabled_encoders()
+    else:
+        packet_encoders = _nodupes(pestr)
+        unknown = [x for x in packet_encoders if x and x not in packet_encoding.ALL_ENCODERS]
+        if unknown:
+            warn("warning: invalid packet encoder(s) specified: %s" % (", ".join(unknown)))
+    for pe in packet_encoding.ALL_ENCODERS:
+        enabled = pe in packet_encoding.get_enabled_encoders() and pe in packet_encoders
+        setattr(packet_encoding, "use_%s" % pe, enabled)
+    #verify that at least one encoder is available:
+    if not packet_encoding.get_enabled_encoders():
+        raise InitException("at least one valid packet encoder must be enabled (not '%s')" % options.packet_encoders)
+
 
 def parse_cmdline(cmdline):
     defaults = make_defaults_struct()
@@ -119,11 +247,16 @@ def do_parse_cmdline(cmdline, defaults):
     if supports_server:
         server_modes.append("start")
         server_modes.append("upgrade")
-        command_options = ["\t%prog start DISPLAY\n",
+        #display: default to required
+        dstr = " DISPLAY"
+        if defaults.displayfd:
+            #display argument is optional (we can use "-displayfd")
+            dstr = " [DISPLAY]"
+        command_options = ["\t%prog start"+dstr+"\n",
                            "\t%prog stop [DISPLAY]\n",
                            "\t%prog exit [DISPLAY]\n",
                            "\t%prog list\n",
-                           "\t%prog upgrade DISPLAY\n",
+                           "\t%prog upgrade [DISPLAY]\n",
                            ] + command_options
     if supports_shadow:
         server_modes.append("shadow")
@@ -131,17 +264,18 @@ def do_parse_cmdline(cmdline, defaults):
     if not supports_server:
         command_options.append("(This xpra installation does not support starting local servers.)")
 
-    parser = OptionParser(version="xpra v%s" % XPRA_VERSION,
+    parser = ModifiedOptionParser(version="xpra v%s" % XPRA_VERSION,
                           usage="\n" + "".join(command_options))
-    hidden_options = {"display" : defaults.display}
+    hidden_options = {"display" : defaults.display,
+                      "displayfd" : defaults.displayfd}
     if len(server_modes):
-        group = OptionGroup(parser, "Server Options",
+        group = optparse.OptionGroup(parser, "Server Options",
                     "These options are only relevant on the server when using the %s mode." %
                     " or ".join(["'%s'" % x for x in server_modes]))
         parser.add_option_group(group)
     if supports_server:
         group.add_option("--start-child", action="append",
-                          dest="start_child", metavar="CMD", default=defaults.start_child,
+                          dest="start_child", metavar="CMD", default=list(defaults.start_child or []),
                           help="program to spawn in new server (may be repeated) (default: %default)")
         group.add_option("--exit-with-children", action="store_true",
                           dest="exit_with_children", default=defaults.exit_with_children,
@@ -197,7 +331,7 @@ def do_parse_cmdline(cmdline, defaults):
         hidden_options["fake_xinerama"] = False
     if supports_server or supports_shadow:
         group.add_option("--bind-tcp", action="append",
-                          dest="bind_tcp", default=defaults.bind_tcp,
+                          dest="bind_tcp", default=list(defaults.bind_tcp or []),
                           metavar="[HOST]:PORT",
                           help="Listen for connections over TCP (use --password-file to secure it)."
                             + " You may specify this option multiple times with different host and port combinations")
@@ -233,7 +367,7 @@ def do_parse_cmdline(cmdline, defaults):
         hidden_options["pulseaudio_command"] = ""
         hidden_options["dbus_proxy"] = False
 
-    group = OptionGroup(parser, "Server Controlled Features",
+    group = optparse.OptionGroup(parser, "Server Controlled Features",
                 "These options can be used to turn certain features on or off, "
                 "they can be specified on the client or on the server, "
                 "but the client cannot enable them if they are disabled on the server.")
@@ -308,7 +442,7 @@ def do_parse_cmdline(cmdline, defaults):
     Use the special value 'help' to get a list of options.
     When unspecified, all the available codecs are allowed and the first one is used."""
         group.add_option("--speaker-codec", action="append",
-                          dest="speaker_codec", default=defaults.speaker_codec,
+                          dest="speaker_codec", default=list(defaults.speaker_codec or []),
                           help=CODEC_HELP % "speaker")
         group.add_option("--microphone", action="store_true",
                           dest="microphone", default=defaults.microphone,
@@ -317,7 +451,7 @@ def do_parse_cmdline(cmdline, defaults):
                           dest="microphone", default=defaults.microphone,
                           help="Disable forwarding of sound input to the server (default: %s)" % enabled_str(defaults.microphone))
         group.add_option("--microphone-codec", action="append",
-                          dest="microphone_codec", default=defaults.microphone_codec,
+                          dest="microphone_codec", default=list(defaults.microphone_codec or []),
                           help=CODEC_HELP % "microphone")
     else:
         hidden_options["speaker"] = False
@@ -325,14 +459,17 @@ def do_parse_cmdline(cmdline, defaults):
         hidden_options["microphone"] = False
         hidden_options["microphone_codec"] = []
 
-    group = OptionGroup(parser, "Picture Encoding and Compression Options",
+    group = optparse.OptionGroup(parser, "Encoding and Compression Options",
                 "These options are used by the client to specify the desired picture and network data compression."
                 "They may also be specified on the server as default values for those clients that do not set them.")
     parser.add_option_group(group)
+    group.add_option("--encodings", action="store",
+                      dest="encodings", default=defaults.encodings,
+                      help="Specify which encodings are allowed (default: %default)")
     group.add_option("--encoding", action="store",
                       metavar="ENCODING", default=defaults.encoding,
                       dest="encoding", type="str",
-                      help="What image compression algorithm to use, specify 'help' to get a list of options."
+                      help="Which image compression algorithm to use, specify 'help' to get a list of options."
                             " Default: %default."
                       )
     if (supports_server or supports_shadow):
@@ -344,6 +481,9 @@ def do_parse_cmdline(cmdline, defaults):
     group.add_option("--csc-modules", action="store",
                       dest="csc_modules", default=defaults.csc_modules,
                       help="Specify which colourspace conversion modules to enable, to get a list of all the options specify 'help' (default: %default)")
+    group.add_option("--video-decoders", action="store",
+                      dest="video_decoders", default=defaults.video_decoders,
+                      help="Specify which video decoders to enable, to get a list of all the options specify 'help'")
     group.add_option("--min-quality", action="store",
                       metavar="MIN-LEVEL",
                       dest="min_quality", type="int", default=defaults.min_quality,
@@ -366,6 +506,12 @@ def do_parse_cmdline(cmdline, defaults):
                       help="Idle delay in seconds before doing an automatic lossless refresh."
                       + " 0.0 to disable."
                       + " Default: %default.")
+    group.add_option("--compressors", action="store",
+                      dest="compressors", default=", ".join(defaults.compressors),
+                      help="The packet compressors to enable (default: %default)")
+    group.add_option("--packet-encoders", action="store",
+                      dest="packet_encoders", default=", ".join(defaults.packet_encoders),
+                      help="The packet encoders to enable (default: %default)")
     group.add_option("-z", "--compress", action="store",
                       dest="compression_level", type="int", default=defaults.compression_level,
                       metavar="LEVEL",
@@ -376,7 +522,7 @@ def do_parse_cmdline(cmdline, defaults):
                       + " 0 to disable compression,"
                       + " 9 for maximal (slowest) compression. Default: %default.")
 
-    group = OptionGroup(parser, "Client Features Options",
+    group = optparse.OptionGroup(parser, "Client Features Options",
                 "These options control client features that affect the appearance or the keyboard.")
     parser.add_option_group(group)
     group.add_option("--opengl", action="store",
@@ -417,7 +563,7 @@ def do_parse_cmdline(cmdline, defaults):
                           dest="tray_icon", default=defaults.tray_icon,
                           help="Path to the image which will be used as icon for the system-tray or dock")
     group.add_option("--key-shortcut", action="append",
-                      dest="key_shortcut", type="str", default=defaults.key_shortcut,
+                      dest="key_shortcut", type="str", default=list(defaults.key_shortcut or []),
                       help="Define key shortcuts that will trigger specific actions."
                       + "If no shortcuts are defined, it defaults to '%s'" % (",".join(defaults.key_shortcut or [])))
     group.add_option("--keyboard-sync", action="store_true",
@@ -427,12 +573,15 @@ def do_parse_cmdline(cmdline, defaults):
                       dest="keyboard_sync", default=defaults.keyboard_sync,
                       help="Disable keyboard state synchronization, prevents keys from repeating on high latency links but also may disrupt applications which access the keyboard directly (default: %s)" % enabled_str(defaults.keyboard_sync))
 
-    group = OptionGroup(parser, "Advanced Options",
+    group = optparse.OptionGroup(parser, "Advanced Options",
                 "These options apply to both client and server. Please refer to the man page for details.")
     parser.add_option_group(group)
     group.add_option("--password-file", action="store",
                       dest="password_file", default=defaults.password_file,
                       help="The file containing the password required to connect (useful to secure TCP mode)")
+    group.add_option("--input-method", action="store",
+                      dest="input_method", default=defaults.input_method,
+                      help="Which X11 input method to configure for client applications started with start-child (default: %default, options: none, keep, xim, IBus, SCIM, uim)")
     group.add_option("--dpi", action="store",
                       dest="dpi", default=defaults.dpi,
                       help="The 'dots per inch' value that client applications should try to honour (default: %default)")
@@ -499,34 +648,14 @@ def do_parse_cmdline(cmdline, defaults):
         categories = options.debug.split(",")
         for cat in categories:
             if cat=="help":
-                print("known logging filters (there may be others): %s" % ", ".join(KNOWN_FILTERS))
-                sys.exit(0)
+                raise InitInfo("known logging filters (there may be others): %s" % ", ".join(KNOWN_FILTERS))
 
-    if options.encoding:
-        #fix old encoding names if needed:
-        from xpra.codecs.loader import ALL_OLD_ENCODING_NAMES_TO_NEW
-        options.encoding = ALL_OLD_ENCODING_NAMES_TO_NEW.get(options.encoding, options.encoding)
-
-    #special case for video encoders and csc, stored as lists, but command line option is a CSV string:
-    if (supports_server or supports_shadow):
-        if type(options.video_encoders)==str:
-            if options.video_encoders=="help":
-                from xpra.codecs.video_helper import ALL_VIDEO_ENCODER_OPTIONS as aveco
-                print("the following video encoders may be available: %s" % ", ".join(aveco))
-                sys.exit(0)
-            elif options.video_encoders=="none":
-                options.video_encoders = []
-            else:
-                options.video_encoders = [x.strip() for x in options.video_encoders.split(",")]
-        if type(options.csc_modules)==str:
-            if options.csc_modules=="help":
-                from xpra.codecs.video_helper import ALL_CSC_MODULE_OPTIONS as acsco
-                print("the following csc modules may be available: %s" % ", ".join(acsco))
-                sys.exit(0)
-            elif options.csc_modules=="none":
-                options.csc_modules = []
-            else:
-                options.csc_modules = [x.strip() for x in options.csc_modules.split(",")]
+    #special case for things stored as lists, but command line option is a CSV string:
+    #and may have "none" or "all" special values
+    fixup_encodings(options)
+    fixup_compression(options)
+    fixup_packetencoding(options)
+    fixup_video_all_or_none(options)
 
     #special handling for URL mode:
     #xpra attach xpra://[mode:]host:port/?param1=value1&param2=value2
@@ -558,19 +687,18 @@ def do_parse_cmdline(cmdline, defaults):
     try:
         int(options.dpi)
     except Exception, e:
-        parser.error("invalid dpi: %s" % e)
+        raise InitException("invalid dpi: %s" % e)
     if options.encryption:
         assert len(ENCRYPTION_CIPHERS)>0, "cannot use encryption: no ciphers available"
         if options.encryption not in ENCRYPTION_CIPHERS:
-            parser.error("encryption %s is not supported, try: %s" % (options.encryption, ", ".join(ENCRYPTION_CIPHERS)))
+            raise InitException("encryption %s is not supported, try: %s" % (options.encryption, ", ".join(ENCRYPTION_CIPHERS)))
         if not options.password_file and not options.encryption_keyfile:
-            parser.error("encryption %s cannot be used without a keyfile (see --encryption-keyfile option)" % options.encryption)
+            raise InitException("encryption %s cannot be used without a keyfile (see --encryption-keyfile option)" % options.encryption)
     #ensure opengl is either True, False or None
     options.opengl = parse_bool("opengl", options.opengl)
-    return parser, options, args
+    return options, args
 
 def dump_frames(*arsg):
-    import traceback
     frames = sys._current_frames()
     print("")
     print("found %s frames:" % len(frames))
@@ -584,8 +712,8 @@ def configure_logging(options, mode):
     if mode in ("start", "upgrade", "attach", "shadow", "proxy"):
         if "help" in options.speaker_codec or "help" in options.microphone_codec:
             from xpra.sound.gstreamer_util import show_sound_codec_help
-            show_sound_codec_help(mode!="attach", options.speaker_codec, options.microphone_codec)
-            return 0
+            info = show_sound_codec_help(mode!="attach", options.speaker_codec, options.microphone_codec)
+            raise InitInfo("\n".join(info))
     else:
         #a bit naughty here, but it's easier to let xpra.log initialize
         #the logging system every time, and just undo things here..
@@ -615,7 +743,7 @@ def configure_logging(options, mode):
         signal.signal(signal.SIGUSR1, sigusr1)
 
 
-def run_mode(script_file, parser, options, args, mode, defaults):
+def run_mode(script_file, error_cb, options, args, mode, defaults):
     #configure default logging handler:
     if os.name=="posix" and os.getuid()==0 and mode!="proxy":
         warn("\nWarning: running as root")
@@ -626,21 +754,21 @@ def run_mode(script_file, parser, options, args, mode, defaults):
         ssh_display = len(args)>0 and (args[0].startswith("ssh/") or args[0].startswith("ssh:"))
         if mode in ("start", "shadow") and ssh_display:
             #ie: "xpra start ssh:HOST:DISPLAY --start-child=xterm"
-            return run_remote_server(parser, options, args, mode, defaults)
+            return run_remote_server(error_cb, options, args, mode, defaults)
         elif (mode in ("start", "upgrade", "proxy") and supports_server) or (mode=="shadow" and supports_shadow):
             nox()
             from xpra.scripts.server import run_server
-            return run_server(parser, options, mode, script_file, args)
-        elif mode in ("attach", "detach", "screenshot", "version", "info", "control"):
-            return run_client(parser, options, args, mode)
+            return run_server(error_cb, options, mode, script_file, args)
+        elif mode in ("attach", "detach", "screenshot", "version", "info", "control", "_monitor"):
+            return run_client(error_cb, options, args, mode)
         elif mode in ("stop", "exit") and (supports_server or supports_shadow):
             nox()
-            return run_stopexit(mode, parser, options, args)
+            return run_stopexit(mode, error_cb, options, args)
         elif mode == "list" and (supports_server or supports_shadow):
-            return run_list(parser, options, args)
+            return run_list(error_cb, options, args)
         elif mode in ("_proxy", "_proxy_start", "_shadow_start") and (supports_server or supports_shadow):
             nox()
-            return run_proxy(parser, options, script_file, args, mode, defaults)
+            return run_proxy(error_cb, options, script_file, args, mode, defaults)
         elif mode == "initenv":
             from xpra.scripts.server import xpra_runner_shell_script, write_runner_shell_script
             script = xpra_runner_shell_script(script_file, os.getcwd(), options.socket_dir)
@@ -648,7 +776,7 @@ def run_mode(script_file, parser, options, args, mode, defaults):
             write_runner_shell_script(dotxpra, script, False)
             return 0
         else:
-            parser.error("invalid mode '%s'" % mode)
+            error_cb("invalid mode '%s'" % mode)
             return 1
     except KeyboardInterrupt, e:
         sys.stderr.write("\ncaught %s, exiting\n" % repr(e))
@@ -786,7 +914,7 @@ def parse_display_name(error_cb, opts, display_name):
     else:
         error_cb("unknown format for display name: %s" % display_name)
 
-def pick_display(parser, opts, extra_args):
+def pick_display(error_cb, opts, extra_args):
     if len(extra_args) == 0:
         # Pick a default server
         sockdir = DotXpra(opts.socket_dir or get_default_socket_dir())
@@ -796,16 +924,16 @@ def pick_display(parser, opts, extra_args):
                         if state is DotXpra.LIVE]
         if len(live_servers) == 0:
             if not LOCAL_SERVERS_SUPPORTED:
-                parser.error("this installation does not support local servers, you must specify a remote display")
-            parser.error("cannot find a live server to connect to")
+                error_cb("this installation does not support local servers, you must specify a remote display")
+            error_cb("cannot find a live server to connect to")
         elif len(live_servers) == 1:
-            return parse_display_name(parser.error, opts, live_servers[0])
+            return parse_display_name(error_cb, opts, live_servers[0])
         else:
-            parser.error("there are multiple servers running, please specify")
+            error_cb("there are multiple servers running, please specify")
     elif len(extra_args) == 1:
-        return parse_display_name(parser.error, opts, extra_args[0])
+        return parse_display_name(error_cb, opts, extra_args[0])
     else:
-        parser.error("too many arguments")
+        error_cb("too many arguments")
 
 def _socket_connect(sock, endpoint, description, dtype):
     sock.connect(endpoint)
@@ -816,7 +944,7 @@ def connect_or_fail(display_desc):
     try:
         return connect_to(display_desc)
     except Exception, e:
-        sys.exit("connection failed: %s" % e)
+        raise InitException("connection failed: %s" % e)
 
 def ssh_connect_failed(message):
     #by the time ssh fails, we may have entered the gtk main loop
@@ -831,7 +959,7 @@ def connect_to(display_desc, debug_cb=None, ssh_fail_cb=ssh_connect_failed):
     if dtype == "ssh":
         cmd = display_desc["full_ssh"]
         proxy_cmd = display_desc["remote_xpra"] + display_desc["proxy_command"] + display_desc["display_as_args"]
-        cmd += ["sh -c 'xpra initenv >> /dev/null 2>&1;"+(" ".join(proxy_cmd))+"'"]
+        cmd += ["sh -c 'xpra initenv >> /dev/null 2>&1 || echo \"Warning: xpra server does not support initenv\" 1>&2;"+(" ".join(proxy_cmd))+"'"]
         try:
             kwargs = {}
             kwargs["stderr"] = sys.stderr
@@ -898,6 +1026,7 @@ def connect_to(display_desc, debug_cb=None, ssh_fail_cb=ssh_connect_failed):
             except Exception, e:
                 print("error trying to stop ssh tunnel process: %s" % e)
         conn = TwoFileConnection(child.stdin, child.stdout, abort_test, target=display_name, info=dtype, close_cb=stop_tunnel)
+        conn.timeout = 0            #taken care of by abort_test
 
     elif dtype == "unix-domain":
         if not hasattr(socket, "AF_UNIX"):
@@ -907,6 +1036,7 @@ def connect_to(display_desc, debug_cb=None, ssh_fail_cb=ssh_connect_failed):
         sock.settimeout(SOCKET_TIMEOUT)
         sockfile = sockdir.socket_path(display_desc["display"])
         conn = _socket_connect(sock, sockfile, display_name, dtype)
+        conn.timeout = SOCKET_TIMEOUT
 
     elif dtype == "tcp":
         sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
@@ -914,26 +1044,27 @@ def connect_to(display_desc, debug_cb=None, ssh_fail_cb=ssh_connect_failed):
         sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, TCP_NODELAY)
         tcp_endpoint = (display_desc["host"], display_desc["port"])
         conn = _socket_connect(sock, tcp_endpoint, display_name, dtype)
+        conn.timeout = SOCKET_TIMEOUT
 
     else:
         assert False, "unsupported display type in connect: %s" % dtype
     return conn
 
 
-def run_client(parser, opts, extra_args, mode):
+def run_client(error_cb, opts, extra_args, mode):
     if mode=="screenshot":
         if len(extra_args)==0:
-            parser.error("invalid number of arguments for screenshot mode")
+            error_cb("invalid number of arguments for screenshot mode")
         screenshot_filename = extra_args[0]
         extra_args = extra_args[1:]
 
     if opts.compression_level < 0 or opts.compression_level > 9:
-        parser.error("Compression level must be between 0 and 9 inclusive.")
+        error_cb("Compression level must be between 0 and 9 inclusive.")
     if opts.quality!=-1 and (opts.quality < 0 or opts.quality > 100):
-        parser.error("Quality must be between 0 and 100 inclusive. (or -1 to disable)")
+        error_cb("Quality must be between 0 and 100 inclusive. (or -1 to disable)")
 
     def connect():
-        return connect_or_fail(pick_display(parser, opts, extra_args))
+        return connect_or_fail(pick_display(error_cb, opts, extra_args))
 
     if mode=="screenshot":
         from xpra.client.gobject_client_base import ScreenshotXpraClient
@@ -941,10 +1072,13 @@ def run_client(parser, opts, extra_args, mode):
     elif mode=="info":
         from xpra.client.gobject_client_base import InfoXpraClient
         app = InfoXpraClient(connect(), opts)
+    elif mode=="_monitor":
+        from xpra.client.gobject_client_base import MonitorXpraClient
+        app = MonitorXpraClient(connect(), opts)
     elif mode=="control":
         from xpra.client.gobject_client_base import ControlXpraClient
         if len(extra_args)<=1:
-            parser.error("not enough arguments for 'control' mode")
+            error_cb("not enough arguments for 'control' mode")
         args = extra_args[1:]
         extra_args = extra_args[:1]
         app = ControlXpraClient(connect(), opts)
@@ -956,33 +1090,30 @@ def run_client(parser, opts, extra_args, mode):
         from xpra.client.gobject_client_base import DetachXpraClient
         app = DetachXpraClient(connect(), opts)
     else:
-        app = make_client(parser.error, opts)
+        app = make_client(error_cb, opts)
         layouts = app.get_supported_window_layouts() or ["default"]
         layouts_str = ", ".join(layouts)
         if opts.window_layout and opts.window_layout.lower()=="help":
-            print("%s supports the following layouts: %s" % (app.client_toolkit(), layouts_str))
-            return 0
+            raise InitInfo("%s supports the following layouts: %s" % (app.client_toolkit(), layouts_str))
         if opts.window_layout and opts.window_layout not in layouts:
-            parser.error("window layout '%s' is not supported by the %s toolkit, valid options are: %s" % (opts.window_layout, app.client_toolkit(), layouts_str))
+            error_cb("window layout '%s' is not supported by the %s toolkit, valid options are: %s" % (opts.window_layout, app.client_toolkit(), layouts_str))
         app.init(opts)
         if opts.encoding:
             #fix old encoding names if needed:
             err = opts.encoding not in app.get_encodings()
+            info = ""
             if err and opts.encoding!="help":
-                print("invalid encoding: %s" % opts.encoding)
+                info = "invalid encoding: %s\n" % opts.encoding
             if opts.encoding=="help" or err:
                 from xpra.codecs.loader import encodings_help
-                print("%s xpra client supports the following encodings:\n * %s" % (app.client_toolkit(), "\n * ".join(encodings_help(app.get_encodings()))))
-                if err:
-                    return 1
-                return 0
+                raise InitInfo(info+"%s xpra client supports the following encodings:\n * %s" % (app.client_toolkit(), "\n * ".join(encodings_help(app.get_encodings()))))
         def handshake_complete(*args):
             from xpra.log import Logger
             log = Logger()
             log.info("Attached to %s (press Control-C to detach)\n" % conn.target)
         if hasattr(app, "connect"):
             app.connect("handshake-complete", handshake_complete)
-        app.init_ui(opts)
+        app.init_ui(opts, extra_args)
         conn = connect()
         app.setup_connection(conn)
     return do_run_client(app)
@@ -1050,9 +1181,9 @@ def strip_defaults_start_child(start_child, defaults_start_child):
                 start_child.remove(x)
     return start_child
 
-def run_remote_server(parser, opts, args, mode, defaults):
+def run_remote_server(error_cb, opts, args, mode, defaults):
     """ Uses the regular XpraClient with patched proxy arguments to tell run_proxy to start the server """
-    params = parse_display_name(parser.error, opts, args[0])
+    params = parse_display_name(error_cb, opts, args[0])
     #add special flags to "display_as_args"
     proxy_args = []
     if params["display"] is not None:
@@ -1066,7 +1197,7 @@ def run_remote_server(parser, opts, args, mode, defaults):
         v = getattr(opts, x.replace("-", "_"))
         if v:
             proxy_args.append("--%s=%s" % (x, v))
-    #thse options must be enabled explicitly (no disable option for most of them):
+    #these options must be enabled explicitly (no disable option for most of them):
     for e in ("exit-with-children", "mmap-group", "readonly"):
         if getattr(opts, e.replace("-", "_")) is True:
             proxy_args.append("--%s" % e)
@@ -1082,7 +1213,7 @@ def run_remote_server(parser, opts, args, mode, defaults):
     else:
         assert mode=="start"
         params["proxy_command"] = ["_proxy_start"]
-    app = make_client(parser.error, opts)
+    app = make_client(error_cb, opts)
     app.init(opts)
     app.init_ui(opts)
     conn = connect_or_fail(params)
@@ -1111,13 +1242,30 @@ def find_X11_displays(max_display_no=10):
                 pass
     return displays
 
-def guess_X11_display():
-    displays = find_X11_displays()
-    assert len(displays)!=0, "could not detect any live X11 displays"
-    assert len(displays)==1, "too many live X11 displays to choose from"
-    return ":%s" % displays[0]
+def guess_xpra_display(socket_dir):
+    sockdir = DotXpra(socket_dir)
+    results = sockdir.sockets()
+    live = [display for state, display in results if state==DotXpra.LIVE]
+    assert len(live)>0, "no existing xpra servers found"
+    assert len(live)<=1, "too many existing xpra servers found, cannot guess which one to use"
+    return live[0]
 
-def run_proxy(parser, opts, script_file, args, mode, defaults):
+def guess_X11_display(socket_dir):
+    displays = [":%s" % x for x in find_X11_displays()]
+    assert len(displays)!=0, "could not detect any live X11 displays"
+    if len(displays)>1:
+        #since we are here to shadow,
+        #assume we want to shadow a real X11 server,
+        #so remove xpra's own displays to narrow things down:
+        sockdir = DotXpra(socket_dir)
+        results = sockdir.sockets()
+        xpra_displays = [display for _, display in results]
+        displays = list(set(displays)-set(xpra_displays))
+        assert len(displays)!=0, "could not detect any live plain X11 displays, only multiple xpra displays: %s" % ", ".join(xpra_displays)
+    assert len(displays)==1, "too many live X11 displays to choose from: %s" % ", ".join(displays)
+    return displays[0]
+
+def run_proxy(error_cb, opts, script_file, args, mode, defaults):
     from xpra.server.proxy import XpraProxy
     assert "gtk" not in sys.modules
     if mode in ("_proxy_start", "_shadow_start"):
@@ -1125,7 +1273,7 @@ def run_proxy(parser, opts, script_file, args, mode, defaults):
         cmd = [script_file]
         if mode=="_proxy_start":
             cmd.append("start")
-            assert len(args)==1, "proxy/shadow-start: expected 1 argument but got %s" % len(args)
+            assert len(args)==1, "_proxy_start: expected 1 argument but got %s: %s" % (len(args), args)
             display_name = args[0]
         else:
             assert mode=="_shadow_start"
@@ -1134,9 +1282,11 @@ def run_proxy(parser, opts, script_file, args, mode, defaults):
                 #display_name was provided:
                 display_name = args[0]
             else:
-                display_name = guess_X11_display()
+                assert len(args)==0, "_shadow_start: expected 0 or 1 arguments but got %s: %s" % (len(args), args)
+                display_name = guess_X11_display(opts.socket_dir)
                 #we now know the display name, so add it:
                 args = [display_name]
+        #adds the display to proxy start command:
         cmd += args
         start_child = strip_defaults_start_child(opts.start_child, defaults.start_child)
         if start_child:
@@ -1162,14 +1312,14 @@ def run_proxy(parser, opts, script_file, args, mode, defaults):
             proc.wait()
         from xpra.daemon_thread import make_daemon_thread
         make_daemon_thread(reaper, "server-startup-reaper").start()
-    server_conn = connect_or_fail(pick_display(parser, opts, args))
+    server_conn = connect_or_fail(pick_display(error_cb, opts, args))
     app = XpraProxy(TwoFileConnection(sys.stdout, sys.stdin, info="stdin/stdout"), server_conn)
     signal.signal(signal.SIGINT, app.quit)
     signal.signal(signal.SIGTERM, app.quit)
     app.run()
     return  0
 
-def run_stopexit(mode, parser, opts, extra_args):
+def run_stopexit(mode, error_cb, opts, extra_args):
     assert "gtk" not in sys.modules
 
     def show_final_state(display):
@@ -1191,7 +1341,7 @@ def run_stopexit(mode, parser, opts, extra_args):
             assert False, "invalid state: %s" % final_state
             return 1
 
-    display_desc = pick_display(parser, opts, extra_args)
+    display_desc = pick_display(error_cb, opts, extra_args)
     conn = connect_or_fail(display_desc)
     e = 1
     try:
@@ -1224,15 +1374,14 @@ def may_cleanup_socket(sockdir, state, display, clean_states=[DotXpra.DEAD]):
             sys.stdout.write(" (cleaned up)")
     sys.stdout.write("\n")
 
-def run_list(parser, opts, extra_args):
+def run_list(error_cb, opts, extra_args):
     assert "gtk" not in sys.modules
     if extra_args:
-        parser.error("too many arguments for mode")
+        error_cb("too many arguments for mode")
     sockdir = DotXpra(opts.socket_dir)
     results = sockdir.sockets()
     if not results:
-        sys.stdout.write("No xpra sessions found\n")
-        return  1
+        error_cb("No xpra sessions found")
     sys.stdout.write("Found the following xpra sessions:\n")
     for state, display in results:
         may_cleanup_socket(sockdir, state, display)
