@@ -10,14 +10,14 @@ from math import sqrt
 from xpra.log import Logger
 log = Logger("window", "encoding")
 
-from xpra.net.protocol import compressed_wrapper, Compressed, use_lz4
+from xpra.net import compression
 try:
     from xpra.codecs.argb.argb import bgra_to_rgb, bgra_to_rgba, argb_to_rgb, argb_to_rgba   #@UnresolvedImport
 except Exception, e:
     log("cannot load argb module: %s", e)
     bgra_to_rgb, bgra_to_rgba, argb_to_rgb, argb_to_rgba = (None,)*4
 from xpra.os_util import StringIOClass
-from xpra.codecs.loader import get_codec, get_codec_version, has_codec
+from xpra.codecs.loader import get_codec, get_codec_version
 from xpra.os_util import builtins
 _memoryview = builtins.__dict__.get("memoryview")
 try:
@@ -28,8 +28,6 @@ except:
 
 PIL = get_codec("PIL")
 PIL_VERSION = get_codec_version("PIL")
-enc_webm = get_codec("enc_webm")
-webp_handlers = get_codec("webm_bitmap_handlers")
 PIL_can_optimize = PIL_VERSION>="2.2"
 
 #give warning message just once per key then ignore:
@@ -63,55 +61,15 @@ def webp_encode(coding, image, supports_transparency, quality, speed, options):
             client_options["quality"] = quality
         if alpha:
             client_options["has_alpha"] = True
-        return "webp", Compressed("webp", cdata), client_options, image.get_width(), image.get_height(), 0, 24
-    enc_webm = get_codec("enc_webm")
-    webp_handlers = get_codec("webm_bitmap_handlers")
-    if enc_webm and webp_handlers:
-        return webm_encode(image, quality)
+        return "webp", compression.Compressed("webp", cdata), client_options, image.get_width(), image.get_height(), 0, 24
     #fallback to PIL
     return PIL_encode(coding, image, quality, speed, supports_transparency)
 
 
-def webm_encode(image, quality):
-    assert enc_webm and webp_handlers, "webp components are missing"
-
-    BitmapHandler = webp_handlers.BitmapHandler
-    handler_encs = {
-                "RGB" : (BitmapHandler.RGB,     "EncodeRGB",  "EncodeLosslessRGB",  False),
-                "BGR" : (BitmapHandler.BGR,     "EncodeBGR",  "EncodeLosslessBGR",  False),
-                "RGBA": (BitmapHandler.RGBA,    "EncodeRGBA", "EncodeLosslessRGBA", True),
-                "RGBX": (BitmapHandler.RGBA,    "EncodeRGBA", "EncodeLosslessRGBA", False),
-                "BGRA": (BitmapHandler.BGRA,    "EncodeBGRA", "EncodeLosslessBGRA", True),
-                "BGRX": (BitmapHandler.BGRA,    "EncodeBGRA", "EncodeLosslessBGRA", False),
-                }
-    pixel_format = image.get_pixel_format()
-    h_e = handler_encs.get(pixel_format)
-    assert h_e is not None, "cannot handle rgb format %s with webp!" % pixel_format
-    bh, lossy_enc, lossless_enc, has_alpha = h_e
-    q = max(1, quality)
-    enc = None
-    if q>=100 and has_codec("enc_webm_lossless"):
-        enc = getattr(enc_webm, lossless_enc)
-        kwargs = {}
-        client_options = {}
-        log("webm_encode(%s, %s) using lossless encoder=%s for %s", image, enc, pixel_format)
-    if enc is None:
-        enc = getattr(enc_webm, lossy_enc)
-        kwargs = {"quality" : q}
-        client_options = {"quality" : q}
-        log("webm_encode(%s, %s) using lossy encoder=%s with quality=%s for %s", image, enc, q, pixel_format)
-    handler = BitmapHandler(image.get_pixels(), bh, image.get_width(), image.get_height(), image.get_rowstride())
-    bpp = 24
-    if has_alpha:
-        client_options["has_alpha"] = True
-        bpp = 32
-    return "webp", Compressed("webp", str(enc(handler, **kwargs).data)), client_options, image.get_width(), image.get_height(), 0, bpp
-
-
 def roundup(n, m):
     return (n + m - 1) & ~(m - 1)
 
-def rgb_encode(coding, image, rgb_formats, supports_transparency, speed, rgb_zlib=True, rgb_lz4=True, encoding_client_options=True, supports_rgb24zlib=True):
+def rgb_encode(coding, image, rgb_formats, supports_transparency, speed, rgb_zlib=True, rgb_lz4=True, rgb_lzo=False, encoding_client_options=True, supports_rgb24zlib=True):
     pixel_format = image.get_pixel_format()
     #log("rgb_encode%s pixel_format=%s, rgb_formats=%s", (coding, image, rgb_formats, supports_transparency, speed, rgb_zlib, rgb_lz4, encoding_client_options, supports_rgb24zlib), pixel_format, rgb_formats)
     if pixel_format not in rgb_formats:
@@ -156,14 +114,24 @@ def rgb_encode(coding, image, rgb_formats, supports_transparency, speed, rgb_zli
     wire_data = raw_data
     level = 0
     algo = "not"
-    if rgb_zlib or rgb_lz4:
+    if len(pixels)>=256 and (rgb_zlib and compression.use_zlib) or (rgb_lz4 and compression.lz4) or (rgb_lzo and compression.use_lzo):
         level = max(0, min(5, int(115-speed)/20))
         if len(pixels)<1024:
             #fewer pixels, make it more likely we won't bother compressing:
             level = level / 2
     if level>0:
-        lz4 = use_lz4 and rgb_lz4 and level<=3
-        wire_data = compressed_wrapper(coding, pixels, level=level, lz4=lz4)
+        if rgb_lz4 and compression.use_lz4:
+            wire_data = compression.compressed_wrapper(coding, pixels, lz4=True)
+            algo = "lz4"
+            level = 1
+        elif rgb_lzo and compression.use_lzo:
+            wire_data = compression.compressed_wrapper(coding, pixels, lzo=True)
+            algo = "lzo"
+            level = 1
+        else:
+            assert rgb_zlib and compression.use_zlib
+            wire_data = compression.compressed_wrapper(coding, pixels, zlib=True, level=level)
+            algo = "zlib"
         raw_data = wire_data.data
         #log("%s/%s data compressed from %s bytes down to %s (%s%%) with lz4=%s",
         #         coding, pixel_format, len(pixels), len(raw_data), int(100.0*len(raw_data)/len(pixels)), self.rgb_lz4)
@@ -173,12 +141,8 @@ def rgb_encode(coding, image, rgb_formats, supports_transparency, speed, rgb_zli
             wire_data = str(pixels)
             raw_data = wire_data
         else:
-            if lz4:
-                options["lz4"] = True
-                algo = "lz4"
-            else:
-                options["zlib"] = level
-                algo = "zlib"
+            #add compressed marker:
+            options[algo] = level
     if pixel_format.upper().find("A")>=0 or pixel_format.upper().find("X")>=0:
         bpp = 32
     else:
@@ -188,7 +152,7 @@ def rgb_encode(coding, image, rgb_formats, supports_transparency, speed, rgb_zli
         return  coding, wire_data, {}, width, height, stride, bpp
     #wrap it using "Compressed" so the network layer receiving it
     #won't decompress it (leave it to the client's draw thread)
-    return coding, Compressed(coding, raw_data), options, width, height, stride, bpp
+    return coding, compression.Compressed(coding, raw_data, True), options, width, height, stride, bpp
 
 
 def PIL_encode(coding, image, quality, speed, supports_transparency):
@@ -227,11 +191,12 @@ def PIL_encode(coding, image, quality, speed, supports_transparency):
         q = int(min(99, max(1, quality)))
         kwargs = im.info
         kwargs["quality"] = q
+        client_options["quality"] = q
         if coding=="jpeg" and PIL_can_optimize and speed<70:
             #(optimizing jpeg is pretty cheap and worth doing)
             kwargs["optimize"] = True
+            client_options["optimize"] = True
         im.save(buf, coding.upper(), **kwargs)
-        client_options["quality"] = q
     else:
         assert coding in ("png", "png/P", "png/L"), "unsupported png encoding: %s" % coding
         if coding in ("png/L", "png/P") and supports_transparency and rgb=="RGBA":
@@ -268,11 +233,14 @@ def PIL_encode(coding, image, quality, speed, supports_transparency):
         if PIL_can_optimize and speed==0:
             #optimizing png is very rarely worth doing
             kwargs["optimize"] = True
+            client_options["optimize"] = True
         #level can range from 0 to 9, but anything above 5 is way too slow for small gains:
         #76-100   -> 1
         #51-76    -> 2
         #etc
-        kwargs["compress_level"] = max(1, min(5, (125-speed)/25))
+        level = max(1, min(5, (125-speed)/25))
+        kwargs["compress_level"] = level
+        client_options["compress_level"] = level
         #default is good enough, no need to override, other options:
         #DEFAULT_STRATEGY, FILTERED, HUFFMAN_ONLY, RLE, FIXED
         #kwargs["compress_type"] = PIL.Image.DEFAULT_STRATEGY
@@ -280,7 +248,7 @@ def PIL_encode(coding, image, quality, speed, supports_transparency):
     log("sending %sx%s %s as %s, mode=%s, options=%s", w, h, pixel_format, coding, im.mode, kwargs)
     data = buf.getvalue()
     buf.close()
-    return coding, Compressed(coding, data), client_options, image.get_width(), image.get_height(), 0, bpp
+    return coding, compression.Compressed(coding, data), client_options, image.get_width(), image.get_height(), 0, bpp
 
 
 def argb_swap(image, rgb_formats, supports_transparency):
