@@ -18,11 +18,13 @@ log = Logger("proxy")
 
 from xpra.server.server_core import get_server_info, get_thread_info
 from xpra.scripts.server import deadly_signal
-from xpra.net.protocol import Protocol, Compressed, compressed_wrapper, new_cipher_caps, get_network_caps
+from xpra.net.compression import Compressed, compressed_wrapper
+from xpra.net.protocol import Protocol, get_network_caps
+from xpra.net.crypto import new_cipher_caps
 from xpra.codecs.image_wrapper import ImageWrapper
 from xpra.codecs.video_helper import getVideoHelper, PREFERRED_ENCODER_ORDER
 from xpra.os_util import Queue, SIGNAMES
-from xpra.util import typedict
+from xpra.util import typedict, updict, LOGIN_TIMEOUT, CONTROL_COMMAND_ERROR, AUTHENTICATION_ERROR, CLIENT_EXIT_TIMEOUT, SERVER_SHUTDOWN
 from xpra.version_util import local_version
 from xpra.daemon_thread import make_daemon_thread
 from xpra.scripts.config import parse_number, parse_bool
@@ -268,14 +270,14 @@ class ProxyInstanceProcess(Process):
     def verify_connection_accepted(self, protocol):
         if not protocol._closed and protocol in self.potential_protocols:
             log.error("connection timedout: %s", protocol)
-            self.send_disconnect(protocol, "login timeout")
+            self.send_disconnect(protocol, LOGIN_TIMEOUT)
 
     def process_control_packet(self, proto, packet):
         try:
             self.do_process_control_packet(proto, packet)
-        except:
+        except Exception, e:
             log.error("error processing control packet", exc_info=True)
-            self.send_disconnect(proto, "error processing request")
+            self.send_disconnect(proto, CONTROL_COMMAND_ERROR, str(e))
 
     def do_process_control_packet(self, proto, packet):
         log("process_control_packet(%s, %s)", proto, packet)
@@ -288,11 +290,11 @@ class ProxyInstanceProcess(Process):
         if packet_type=="hello":
             caps = typedict(packet[1])
             if caps.boolget("challenge"):
-                self.send_disconnect(proto, "this socket does not use authentication")
+                self.send_disconnect(proto, AUTHENTICATION_ERROR, "this socket does not use authentication")
                 return
             if caps.get("info_request", False):
                 proto.send_now(("hello", self.get_proxy_info(proto)))
-                self.timeout_add(5*1000, self.send_disconnect, proto, "info sent")
+                self.timeout_add(5*1000, self.send_disconnect, proto, CLIENT_EXIT_TIMEOUT, "info sent")
                 return
             elif caps.get("stop_request", False):
                 self.stop("socket request", None)
@@ -300,15 +302,15 @@ class ProxyInstanceProcess(Process):
             elif caps.get("version_request", False):
                 from xpra import __version__
                 proto.send_now(("hello", {"version" : __version__}))
-                self.timeout_add(5*1000, self.send_disconnect, proto, "version sent")
+                self.timeout_add(5*1000, self.send_disconnect, proto, CLIENT_EXIT_TIMEOUT, "version sent")
                 return
-        self.send_disconnect(proto, "this socket only handles 'info', 'version' and 'stop' requests")
+        self.send_disconnect(proto, CONTROL_COMMAND_ERROR, "this socket only handles 'info', 'version' and 'stop' requests")
 
-    def send_disconnect(self, proto, reason):
-        log("send_disconnect(%s, %s)", proto, reason)
+    def send_disconnect(self, proto, reason, *extra):
+        log("send_disconnect(%s, %s, %s)", proto, reason, extra)
         if proto._closed:
             return
-        proto.send_now(["disconnect", reason])
+        proto.send_now(["disconnect", reason]+list(extra))
         self.timeout_add(1000, self.force_disconnect, proto)
 
     def force_disconnect(self, proto):
@@ -317,8 +319,10 @@ class ProxyInstanceProcess(Process):
 
     def get_proxy_info(self, proto):
         info = {"proxy.version" : local_version}
-        info.update(get_server_info("proxy"))
-        info.update(get_thread_info("proxy", proto))
+        def upp(d):
+            updict(info, "proxy", d)
+        upp(get_server_info())
+        upp(get_thread_info(proto))
         info.update(self.get_encoder_info())
         return info
 
@@ -328,7 +332,12 @@ class ProxyInstanceProcess(Process):
         def number(k, v):
             return parse_number(int, k, v)
         OPTION_WHITELIST = {"compression_level" : number,
-                            "lz4"               : parse_bool}
+                            "lz4"               : parse_bool,
+                            "lzo"               : parse_bool,
+                            "zlib"              : parse_bool,
+                            "rencode"           : parse_bool,
+                            "bencode"           : parse_bool,
+                            "yaml"              : parse_bool}
         for k,v in options.items():
             parser = OPTION_WHITELIST.get(k)
             if parser:
@@ -352,6 +361,8 @@ class ProxyInstanceProcess(Process):
     def filter_server_caps(self, caps):
         if caps.get("rencode", False):
             self.server_protocol.enable_rencode()
+        elif caps.get("yaml", False):
+            self.server_protocol.enable_yaml()
         return self.filter_caps(caps, ("aliases", ))
 
     def filter_caps(self, caps, prefixes):
@@ -369,7 +380,7 @@ class ProxyInstanceProcess(Process):
         #replace the network caps with the proxy's own:
         pcaps.update(get_network_caps())
         #then add the proxy info:
-        pcaps.update(get_server_info("proxy"))
+        updict(pcaps, "proxy", get_server_info())
         pcaps["proxy"] = True
         pcaps["proxy.hostname"] = socket.gethostname()
         return pcaps
@@ -393,6 +404,13 @@ class ProxyInstanceProcess(Process):
             except:
                 log.error("error during main loop callback %s", fn, exc_info=True)
         self.exit = True
+        #wait for connections to close down cleanly before we exit
+        for i in range(10):
+            if self.client_protocol._closed and self.server_protocol._closed:
+                break
+            if i==0:
+                log.info("waiting for network connections to close")
+            time.sleep(0.1)
         log.info("proxy instance %s stopped", os.getpid())
 
     def stop(self, reason="proxy terminating", skip_proto=None):
@@ -414,7 +432,7 @@ class ProxyInstanceProcess(Process):
         self.encode_queue = q
         for proto in (self.client_protocol, self.server_protocol):
             if proto and proto!=skip_proto:
-                proto.flush_then_close(["disconnect", reason])
+                proto.flush_then_close(["disconnect", SERVER_SHUTDOWN, reason])
 
 
     def queue_client_packet(self, packet):
@@ -491,6 +509,8 @@ class ProxyInstanceProcess(Process):
             self.encode_queue.put(packet)
             #which will queue the packet itself when done:
             return
+        #we do want to reformat cursor packets...
+        #as they will have been uncompressed by the network layer already:
         elif packet_type=="cursor":
             #packet = ["cursor", x, y, width, height, xhot, yhot, serial, pixels, name]
             #or:
@@ -500,7 +520,15 @@ class ProxyInstanceProcess(Process):
                 if len(pixels)<512:
                     packet[8] = str(pixels)
                 else:
-                    packet[8] = compressed_wrapper("cursor", pixels)
+                    #FIXME: this is ugly and not generic!
+                    zlib = self.caps.get("zlib", True)
+                    lz4 = self.caps.get("lz4", False)
+                    lzo = self.caps.get("lzo", False)
+                    if zlib or lz4 or lzo:
+                        packet[8] = compressed_wrapper("cursor", pixels, zlib=zlib, lz4=lz4, lzo=lzo, can_inline=False)
+                    else:
+                        #prevent warnings about large uncompressed data
+                        packet[8] = Compressed("raw cursor", pixels, can_inline=True)
         self.queue_client_packet(packet)
 
 
