@@ -18,56 +18,61 @@ import thread
 
 from xpra.log import Logger
 log = Logger("server")
+commandlog = Logger("command")
 
 import xpra
+from xpra.server import ClientException
 from xpra.scripts.main import SOCKET_TIMEOUT, _socket_connect
 from xpra.scripts.config import ENCRYPTION_CIPHERS
 from xpra.scripts.server import deadly_signal
 from xpra.net.bytestreams import SocketConnection
 from xpra.platform import set_application_name
 from xpra.os_util import load_binary_file, get_machine_id, get_user_uuid, SIGNAMES
-from xpra.version_util import version_compat_check, get_version_info, get_platform_info, get_host_info, local_version, mk
-from xpra.net.protocol import Protocol, use_lz4, use_rencode, new_cipher_caps, get_network_caps, repr_ellipsized
+from xpra.version_util import version_compat_check, get_version_info, get_platform_info, get_host_info, local_version
+from xpra.net.protocol import Protocol, get_network_caps
+from xpra.net.crypto import new_cipher_caps
 from xpra.server.background_worker import stop_worker
 from xpra.daemon_thread import make_daemon_thread
 from xpra.server.proxy import XpraProxy
-from xpra.util import typedict
-
+from xpra.util import typedict, updict, repr_ellipsized, \
+        SERVER_SHUTDOWN, SERVER_EXIT, LOGIN_TIMEOUT, DONE, PROTOCOL_ERROR, SERVER_ERROR, VERSION_ERROR, AUTHENTICATION_ERROR
 
 
 MAX_CONCURRENT_CONNECTIONS = 20
 
 
-def get_server_info(prefix=""):
+def get_server_info():
     #this function is for non UI thread info
     info = {}
-    info.update(get_host_info(prefix))
-    info.update(get_platform_info(mk(prefix, "platform")))
-    info.update(get_version_info(mk(prefix, "build")))
+    info.update(get_host_info())
+    def up(prefix, d):
+        updict(info, prefix, d)
+    up("platform",  get_platform_info())
+    up("build",     get_version_info())
     return info
 
-def get_thread_info(prefix="", proto=None):
+def get_thread_info(proto=None):
     #threads:
-    info = {}
     info_threads = proto.get_threads()
-    info[prefix+"threads"] = threading.active_count() - len(info_threads)
-    info[prefix+"info_threads"] = len(info_threads)
+    info = {
+            "count"        : threading.activeCount() - len(info_threads),
+            "info.count"   : len(info_threads)
+            }
     i = 0
     #threads used by the "info" client:
     for t in info_threads:
-        info[prefix+"info_thread[%s]" % i] = t.name
+        info["info[%s]" % i] = t.getName()
         i += 1
     i = 0
     #all non-info threads:
     for t in threading.enumerate():
         if t not in info_threads:
-            info[prefix+"thread[%s]" % i] = t.name
+            info[str(i)] = t.getName()
             i += 1
     #platform specific bits:
     try:
         from xpra.platform.info import get_sys_info
-        for k,v in get_sys_info().items():
-            info[prefix+k] = v
+        info.update(get_sys_info())
     except:
         log.error("error getting system info", exc_info=True)
     return info
@@ -88,6 +93,7 @@ class ServerCore(object):
         self.start_time = time.time()
         self.auth_class = None
         self._when_ready = []
+        self.child_reaper = None
 
         self._upgrading = False
         #networking bits:
@@ -195,9 +201,9 @@ class ServerCore(object):
         log("initializing packet handlers")
         self._default_packet_handlers = {
             "hello":                                self._process_hello,
-            "info-request":                         self._process_info_request,
             Protocol.CONNECTION_LOST:               self._process_connection_lost,
             Protocol.GIBBERISH:                     self._process_gibberish,
+            Protocol.INVALID:                       self._process_invalid,
             }
 
     def init_aliases(self):
@@ -210,6 +216,10 @@ class ServerCore(object):
             self._reverse_aliases[key] = i
             i += 1
 
+
+    def reaper_quit(self):
+        self.clean_quit(False)
+
     def signal_quit(self, signum, frame):
         log.info("")
         log.info("got signal %s, exiting", SIGNAMES.get(signum, signum))
@@ -218,6 +228,13 @@ class ServerCore(object):
         self.clean_quit(True)
 
     def clean_quit(self, from_signal=False, upgrading=False):
+        log("clean_quit(%s, %s)", from_signal, upgrading)
+        #ensure the reaper doesn't call us again:
+        if self.child_reaper:
+            def noop():
+                pass
+            self.reaper_quit = noop
+            log("clean_quit: reaper_quit=%s", self.reaper_quit)
         self.cleanup()
         def quit_timer(*args):
             log.debug("quit_timer()")
@@ -230,8 +247,10 @@ class ServerCore(object):
         self.timeout_add(500, quit_timer)
         def force_quit(*args):
             log.debug("force_quit()")
-            os._exit(1)
+            from xpra import os_util
+            os_util.force_quit()
         self.timeout_add(5000, force_quit)
+        log("clean_quit(..) quit timers scheduled")
 
     def quit(self, upgrading):
         log("quit(%s)", upgrading)
@@ -239,6 +258,7 @@ class ServerCore(object):
         log.info("xpra is terminating.")
         sys.stdout.flush()
         self.do_quit()
+        log("quit(%s) do_quit done!", upgrading)
 
     def do_quit(self):
         raise NotImplementedError()
@@ -279,9 +299,9 @@ class ServerCore(object):
             p.quit()
         log("cleanup will disconnect: %s", self._potential_protocols)
         if self._upgrading:
-            reason = "upgrading/exiting"
+            reason = SERVER_EXIT
         else:
-            reason = "shutting down"
+            reason = SERVER_SHUTDOWN
         for proto in list(self._potential_protocols):
             self.disconnect_client(proto, reason)
         self._potential_protocols = []
@@ -319,13 +339,9 @@ class ServerCore(object):
     def invalid_header(self, proto, data):
         log("invalid_header(%s, %s)", proto, repr_ellipsized(data))
         if proto.input_packetcount==0 and self._tcp_proxy:
-            #look for http get:
-            if data[:4]=="GET ":
-                self.start_tcp_proxy(proto, data)
-                return
-        err = "invalid packet header byte: '%#x', not an xpra client?" % ord(data[0])
-        if len(data)>1:
-            err += " read buffer=0x%s" % repr_ellipsized(data)
+            self.start_tcp_proxy(proto, data)
+            return
+        err = "invalid packet format, not an xpra client?"
         proto.gibberish(err, data)
 
     def start_tcp_proxy(self, proto, data):
@@ -363,60 +379,70 @@ class ServerCore(object):
     def verify_connection_accepted(self, protocol):
         if self.is_timedout(protocol):
             log.error("connection timedout: %s", protocol)
-            self.send_disconnect(protocol, "login timeout")
+            self.send_disconnect(protocol, LOGIN_TIMEOUT)
 
-    def send_disconnect(self, proto, reason):
-        log("send_disconnect(%s, %s)", proto, reason)
+    def send_disconnect(self, proto, reason, *extra):
+        log("send_disconnect(%s, %s, %s)", proto, reason, extra)
         if proto._closed:
             return
-        proto.send_now(["disconnect", reason])
+        proto.send_now(["disconnect", reason]+list(extra))
         self.timeout_add(1000, self.force_disconnect, proto)
 
     def force_disconnect(self, proto):
         proto.close()
 
-    def disconnect_client(self, protocol, reason):
-        if protocol:
-            self.disconnect_protocol(protocol, reason)
-        log.info("Connection lost")
+    def disconnect_client(self, protocol, reason, *extra):
+        if protocol and not protocol._closed:
+            self.disconnect_protocol(protocol, reason, *extra)
 
-    def disconnect_protocol(self, protocol, reason):
-        log.info("Disconnecting existing client %s, reason is: %s", protocol, reason)
-        protocol.flush_then_close(["disconnect", reason])
+    def disconnect_protocol(self, protocol, reason, *extra):
+        i = str(reason)
+        if extra:
+            i += " (%s)" % extra
+        log.info("Disconnecting client %s: %s", protocol, i)
+        protocol.flush_then_close(["disconnect", reason]+list(extra))
 
 
     def _process_connection_lost(self, proto, packet):
-        log.info("Connection lost")
         if proto in self._potential_protocols:
+            log.info("Connection lost")
             self._potential_protocols.remove(proto)
 
     def _process_gibberish(self, proto, packet):
-        data = packet[1]
-        log.info("Received uninterpretable nonsense: %s", repr(data))
-        self.disconnect_client(proto, "invalid packet format, not an xpra client?")
+        (_, message, data) = packet
+        log("Received uninterpretable nonsense from %s: %s", proto, message)
+        log(" data: %s", repr_ellipsized(data))
+        self.disconnect_client(proto, message)
+
+    def _process_invalid(self, protocol, packet):
+        (_, message, data) = packet
+        log("Received invalid packet: %s", message)
+        log(" data: %s", repr_ellipsized(data))
+        self.disconnect_client(protocol, message)
 
 
     def send_version_info(self, proto):
         response = {"version" : xpra.__version__}
         proto.send_now(("hello", response))
-        self.timeout_add(5*1000, self.send_disconnect, proto, "version sent")
+        #client is meant to close the connection itself, but just in case:
+        self.timeout_add(5*1000, self.send_disconnect, proto, DONE, "version sent")
 
     def _process_hello(self, proto, packet):
         capabilities = packet[1]
         c = typedict(capabilities)
         proto.set_compression_level(c.intget("compression_level", self.compression_level))
-        if use_rencode and c.boolget("rencode"):
-            proto.enable_rencode()
-        else:
-            proto.enable_bencode()
-
-        if c.boolget("lz4") and use_lz4 and self.compression_level==1:
-            proto.enable_lz4()
+        proto.enable_compressor_from_caps(c)
+        if not proto.enable_encoder_from_caps(c):
+            #this should never happen:
+            #if we got here, we parsed a packet from the client!
+            #(maybe the client used an encoding it claims not to support?)
+            self.disconnect_client(proto, PROTOCOL_ERROR, "failed to negotiate a packet encoder")
+            return
 
         log("process_hello: capabilities=%s", capabilities)
         if c.boolget("version_request"):
             self.send_version_info(proto)
-            return False
+            return
 
         auth_caps = self.verify_hello(proto, c)
         if auth_caps is not False:
@@ -431,9 +457,15 @@ class ServerCore(object):
             #continue processing hello packet:
             try:
                 self.hello_oked(proto, packet, c, auth_caps)
-            except:
-                log.error("server error processing new connection from %s", proto, exc_info=True)
-                self.disconnect_client(proto, "server error accepting new connection")
+            except ClientException, e:
+                log.error("error setting up connection for %s: %s", proto, e)
+                self.disconnect_client(proto, SERVER_ERROR, str(e))
+            except Exception, e:
+                #log full stack trace at debug level,
+                #log exception as error
+                #but don't disclose internal details to the client
+                log.error("server error processing new connection from %s: %s", proto, e, exc_info=True)
+                self.disconnect_client(proto, SERVER_ERROR, "error accepting new connection")
 
     def set_socket_timeout(self, conn, timeout=None):
         #FIXME: this is ugly, but less intrusive than the alternative?
@@ -445,7 +477,7 @@ class ServerCore(object):
         remote_version = c.strget("version")
         verr = version_compat_check(remote_version)
         if verr is not None:
-            self.disconnect_client(proto, "incompatible version: %s" % verr)
+            self.disconnect_client(proto, VERSION_ERROR, "incompatible version: %s" % verr)
             proto.close()
             return  False
 
@@ -512,7 +544,7 @@ class ServerCore(object):
         else:
             #did the client expect a challenge?
             if c.boolget("challenge"):
-                self.disconnect_client(proto, "this server does not require authentication")
+                self.disconnect_client(proto, AUTHENTICATION_ERROR, "this server does not require authentication")
                 return False
         return auth_caps
 
@@ -541,30 +573,36 @@ class ServerCore(object):
         pass
 
 
-    def control_command_response(self, proto, command, error=0, response=""):
-        log("control_command_response(%s)=%s", command, response)
-        hello = {"command_response"  : (error, response)}
-        proto.send_now(("hello", hello))
-
     def handle_command_request(self, proto, args):
+        """ client sent a command request as part of the hello packet """
         try:
             assert len(args)>0
             command = args[0]
+            error = 0
             if command not in self.control_commands:
-                log.warn("invalid command: %s (must be one of: %s)", command, self.control_commands)
-                return self.control_command_response(proto, command, 6, "invalid command")
-            self.do_handle_command_request(proto, command, args[1:])
+                commandlog.warn("invalid command: %s (must be one of: %s)", command, self.control_commands)
+                error = 6
+                response = "invalid command"
+            else:
+                error, response = self.do_handle_command_request(command, args[1:])
         except Exception, e:
-            log.error("error processing command %s", args, exc_info=True)
-            proto.send_now(("hello", {"command_response"  : (127, "error processing command: %s" % e)}))
+            commandlog.error("error processing command %s", command, exc_info=True)
+            error = 127
+            response = "error processing command: %s" % e
+        hello = {"command_response"  : (error, response)}
+        proto.send_now(("hello", hello))
 
-    def do_handle_command_request(self, proto, command, args):
-        log("handle_command_request(%s, %s, %s)", proto, command, args)
+    def do_handle_command_request(self, command, args):
+        """ this may get called by:
+            * handle_command_request from a hello packet
+            * _process_command_request from a dedicated packet
+            it is overriden in subclasses.
+        """
         if command=="hello":
-            return self.control_command_response(proto, command, 0, "hello")
-        assert command=="help"
-        return self.control_command_response(proto, command, 0,
-                    "control supports: %s" % (", ".join(self.control_commands)))
+            return 0, "hello"
+        elif command=="help":
+            return 0, "control supports: %s" % (", ".join(self.control_commands))
+        return 9, "invalid command '%s'" % command
 
 
     def accept_client(self, proto, c):
@@ -574,26 +612,27 @@ class ServerCore(object):
         if proto in self._potential_protocols:
             self._potential_protocols.remove(proto)
 
-    def make_hello(self):
+    def make_hello(self, source):
         now = time.time()
         capabilities = get_network_caps()
-        capabilities.update(get_server_info())
+        if source.wants_versions:
+            capabilities.update(get_server_info())
         capabilities.update({
                         "version"               : xpra.__version__,
                         "start_time"            : int(self.start_time),
                         "current_time"          : int(now),
                         "elapsed_time"          : int(now - self.start_time),
                         "server_type"           : "core",
-                        "info-request"          : True,
-                        "uuid"                  : get_user_uuid(),
                         })
-        mid = get_machine_id()
-        if mid:
-            capabilities["machine_id"] = mid
+        if source.wants_features:
+            capabilities["info-request"] = True
+        if source.wants_versions:
+            capabilities["uuid"] = get_user_uuid()
+            mid = get_machine_id()
+            if mid:
+                capabilities["machine_id"] = mid
         if self.session_name:
             capabilities["session_name"] = self.session_name
-        if self._reverse_aliases:
-            capabilities["aliases"] = self._reverse_aliases
         return capabilities
 
 
@@ -606,14 +645,6 @@ class ServerCore(object):
     def do_send_info(self, proto, info):
         proto.send_now(("hello", info))
 
-    def _process_info_request(self, proto, packet):
-        ss = self._server_sources.get(proto)
-        assert ss, "cannot find server source for %s" % proto
-        def info_callback(_proto, info):
-            assert proto==_proto
-            ss.send_info_response(info)
-        self.get_all_info(info_callback, proto, *packet[1:])
-
     def get_all_info(self, callback, proto, *args):
         ui_info = self.get_ui_info(proto, *args)
         def in_thread(*args):
@@ -632,44 +663,23 @@ class ServerCore(object):
 
     def get_info(self, proto, *args):
         #this function is for non UI thread info
-        info = get_server_info("server")
-        info.update({
-                "server.mode"               : self.get_server_mode(),
-                "server.type"               : "Python",
-                "server.start_time"         : int(self.start_time),
-                "server.authenticator"      : str((self.auth_class or str)("")),
+        info = {}
+        def up(prefix, d):
+            updict(info, prefix, d)
+
+        up("network",   get_network_caps())
+        up("server",    get_server_info())
+        up("threads",   get_thread_info(proto))
+        up("server", {
+                "mode"              : self.get_server_mode(),
+                "type"              : "Python",
+                "start_time"        : int(self.start_time),
+                "authenticator"     : str((self.auth_class or str)("")),
                 })
         if self.session_name:
             info["session.name"] = self.session_name
-        info.update(self.get_thread_info(proto))
-        for k,v in get_network_caps().items():
-            info["network."+k] = v
-        return info
-
-    def get_thread_info(self, proto):
-        #threads:
-        info = {}
-        info_threads = proto.get_threads()
-        info["threads"] = threading.active_count() - len(info_threads)
-        info["info_threads"] = len(info_threads)
-        i = 0
-        #threads used by the "info" client:
-        for t in info_threads:
-            info["info_thread[%s]" % i] = t.name
-            i += 1
-        i = 0
-        #all non-info threads:
-        for t in threading.enumerate():
-            if t not in info_threads:
-                info["thread[%s]" % i] = t.name
-                i += 1
-        #platform specific bits:
-        try:
-            from xpra.platform.info import get_sys_info
-            for k,v in get_sys_info().items():
-                info[k] = v
-        except:
-            log.error("error getting system info", exc_info=True)
+        if self.child_reaper:
+            info.update(self.child_reaper.get_info())
         return info
 
     def process_packet(self, proto, packet):
@@ -683,8 +693,7 @@ class ServerCore(object):
                 handler(proto, packet)
                 return
             log.error("unknown or invalid packet type: %s from %s", packet_type, proto)
-            if proto not in self._server_sources:
-                proto.close()
+            proto.close()
         except KeyboardInterrupt:
             raise
         except:
