@@ -9,9 +9,9 @@ import socket
 from xpra.log import Logger
 log = Logger("shadow")
 
-from xpra.net.protocol import Compressed
+from xpra.net.compression import Compressed
 from xpra.server.batch_config import DamageBatchConfig
-from xpra.util import AdHocStruct, prettify_plug_name
+from xpra.util import AdHocStruct, prettify_plug_name, DONE
 
 DEFAULT_DELAY = 50              #50ms refresh
 
@@ -104,7 +104,7 @@ class ShadowServerBase(object):
     def get_server_mode(self):
         return "shadow"
 
-    def make_hello(self):
+    def make_hello(self, source):
         return {"shadow" : True}
 
 
@@ -162,8 +162,8 @@ class ShadowServerBase(object):
     def makeRootWindowModel(self):
         return RootWindowModel(self.root)
 
-    def send_windows_and_cursors(self, ss):
-        log("send_windows_and_cursors(%s) will send: %s", ss, self._id_to_window)
+    def send_windows_and_cursors(self, ss, sharing=False):
+        log("send_windows_and_cursors(%s, %s) will send: %s", ss, sharing, self._id_to_window)
         for wid in sorted(self._id_to_window.keys()):
             window = self._id_to_window[wid]
             assert window == self.root_window_model
@@ -214,7 +214,7 @@ class ShadowServerBase(object):
     def _process_close_window(self, proto, packet):
         wid = packet[1]
         self._process_window_common(wid)
-        self.disconnect_client(proto, "closed the only window")
+        self.disconnect_client(proto, DONE, "closed the only window")
 
     def make_screenshot_packet(self):
         w, h, encoding, rowstride, data = self.root_window_model.take_screenshot()
