@@ -17,21 +17,23 @@ log = Logger("server")
 elog = Logger("encoding")
 soundlog = Logger("sound")
 keylog = Logger("keyboard")
+cursorlog = Logger("cursor")
 
+from xpra.server import ClientException
 from xpra.server.source_stats import GlobalPerformanceStatistics
 from xpra.server.window_source import WindowSource, HAS_ALPHA
 from xpra.server.window_video_source import WindowVideoSource
 from xpra.server.batch_config import DamageBatchConfig
 from xpra.simple_stats import add_list_stats, std_unit
-from xpra.scripts.config import python_platform
 from xpra.codecs.loader import get_codec, has_codec, OLD_ENCODING_NAMES_TO_NEW, NEW_ENCODING_NAMES_TO_OLD
 from xpra.codecs.video_helper import getVideoHelper
 from xpra.codecs.codec_constants import codec_spec
-from xpra.net.protocol import compressed_wrapper, Compressed
+from xpra.net import compression
+from xpra.net.compression import compressed_wrapper, Compressed, Uncompressed
 from xpra.daemon_thread import make_daemon_thread
 from xpra.os_util import platform_name, StringIOClass, thread, Queue, get_machine_id, get_user_uuid
 from xpra.server.background_worker import add_work_item
-from xpra.util import std, typedict
+from xpra.util import std, typedict, updict, get_screen_info, CLIENT_PING_TIMEOUT
 
 
 NOYIELD = os.environ.get("XPRA_YIELD") is None
@@ -173,13 +175,13 @@ class ServerSource(object):
     It sends damage pixels to the client via its 'protocol' instance (network connection).
 
     Strategy: if we have 'ordinary_packets' to send, send those.
-    When we don't, then send window updates from the 'damage_packet_queue'.
+    When we don't, then send packets from the 'packet_queue'. (compressed pixels or clipboard data)
     See 'next_packet'.
 
     The UI thread calls damage(), which goes into WindowSource and eventually (batching may be involved)
-    adds the damage pixels ready for processing to the damage_data_queue,
-    items are picked off by the separate 'data_to_packet' thread and added to the
-    damage_packet_queue.
+    adds the damage pixels ready for processing to the compression_work_queue,
+    items are picked off by the separate 'encode' thread (see )
+    and added to the damage_packet_queue.
     """
 
     def __init__(self, protocol, disconnect_cb, idle_add, timeout_add, source_remove,
@@ -239,12 +241,13 @@ class ServerSource(object):
         self.connection_time = time.time()
 
         # the queues of damage requests we work through:
-        self.damage_data_queue = Queue()           #holds functions to call to process damage data
-                                                    #items placed in this queue are picked off by the "data_to_packet" thread,
-                                                    #the functions should add the packets they generate to the 'damage_packet_queue'
-        self.damage_packet_queue = deque()         #holds actual packets ready for sending (already encoded)
+        self.compression_work_queue = Queue()       #holds functions to call to compress data (pixels, clipboard)
+                                                    #items placed in this queue are picked off by the "encode" thread,
+                                                    #the functions should add the packets they generate to the 'packet_queue'
+        self.packet_queue = deque()                 #holds actual packets ready for sending (already encoded)
                                                     #these packets are picked off by the "protocol" via 'next_packet()'
                                                     #format: packet, wid, pixels, start_send_cb, end_send_cb
+                                                    #(only packet is required - the rest can be 0/None for clipboard packets)
         #if we "proxy video", we will modify the video helper to add
         #new encoders, so we must make a deep copy to preserve the original
         #which may be used by other clients (other ServerSource instances)
@@ -258,7 +261,7 @@ class ServerSource(object):
 
         # ready for processing:
         protocol.set_packet_source(self.next_packet)
-        self.encode_thread = make_daemon_thread(self.data_to_packet, "encode")
+        self.encode_thread = make_daemon_thread(self.encode_loop, "encode")
         self.encode_thread.start()
         #for managing the recalculate_delays work:
         self.calculate_window_ids = set()
@@ -319,6 +322,15 @@ class ServerSource(object):
         self.notify_startup_complete = False
         self.control_commands = []
         self.supports_transparency = False
+        #what we send back in hello packet:
+        self.ui_client = True
+        self.wants_aliases = True
+        self.wants_encodings = True
+        self.wants_versions = True
+        self.wants_features = True
+        self.wants_display = True
+        self.wants_sound = True
+        self.wants_events = False
         #sound props:
         self.pulseaudio_id = None
         self.pulseaudio_server = None
@@ -327,8 +339,8 @@ class ServerSource(object):
         self.server_driven = False
 
         self.keyboard_config = None
-        self.cursor_data = None
         self.send_cursor_pending = False
+        self.last_cursor_sent = None
 
 
     def is_closed(self):
@@ -343,7 +355,7 @@ class ServerSource(object):
         #it is now safe to add the end of queue marker:
         #(all window sources will have stopped queuing data,
         # they queue data using the UI thread, so we use it too)
-        self.idle_add(self.damage_data_queue.put, None)
+        self.idle_add(self.compression_work_queue.put, None)
         #this should be a noop since we inherit an initialized helper:
         self.video_helper.cleanup()
         if self.mmap:
@@ -477,6 +489,15 @@ class ServerSource(object):
                 v = min(maxv, v)
             assert v is not None
             return v
+        self.ui_client = c.boolget("ui_client", True)
+        self.wants_encodings = c.boolget("wants_encodings", self.ui_client)
+        self.wants_display = c.boolget("wants_display", self.ui_client)
+        self.wants_sound = c.boolget("wants_sound", True)
+        self.wants_events = c.boolget("wants_events", False)
+        self.wants_aliases = c.boolget("wants_aliases", True)
+        self.wants_versions = c.boolget("wants_versions", True)
+        self.wants_features = c.boolget("wants_features", True)
+
         self.default_batch_config.always = bool(batch_value("always", DamageBatchConfig.ALWAYS))
         self.default_batch_config.min_delay = batch_value("min_delay", DamageBatchConfig.MIN_DELAY, 0, 1000)
         self.default_batch_config.max_delay = batch_value("max_delay", DamageBatchConfig.MAX_DELAY, 1, 15000)
@@ -499,8 +520,10 @@ class ServerSource(object):
         self.client_version = c.strget("version")
         self.client_proxy = c.boolget("proxy")
         #general features:
-        self.lz4 = c.boolget("lz4", False)
-        self.send_windows = c.boolget("windows", True)
+        self.zlib = c.boolget("zlib", True)
+        self.lz4 = c.boolget("lz4", False) and compression.use_lz4
+        self.lzo = c.boolget("lzo", False) and compression.use_lzo
+        self.send_windows = self.ui_client and c.boolget("windows", True)
         self.window_raise = c.boolget("window.raise")
         self.pointer_grabs = c.boolget("pointer.grabs")
         self.server_window_moveresize = c.boolget("server-window-move-resize")
@@ -909,9 +932,9 @@ class ServerSource(object):
         if not self.is_closed():
             if len(self.ordinary_packets)>0:
                 packet = self.ordinary_packets.pop(0)
-            elif len(self.damage_packet_queue)>0:
-                packet, _, _, start_send_cb, end_send_cb = self.damage_packet_queue.popleft()
-            have_more = packet is not None and (len(self.ordinary_packets)>0 or len(self.damage_packet_queue)>0)
+            elif len(self.packet_queue)>0:
+                packet, _, _, start_send_cb, end_send_cb = self.packet_queue.popleft()
+            have_more = packet is not None and (len(self.ordinary_packets)>0 or len(self.packet_queue)>0)
         return packet, start_send_cb, end_send_cb, have_more
 
     def send(self, *parts):
@@ -925,10 +948,21 @@ class ServerSource(object):
 # Functions used by the server to request something
 # (window events, stats, user requests, etc)
 #
-    def set_encoding(self, encoding, window_ids):
+    def set_auto_refresh_delay(self, delay, window_ids):
+        if window_ids is not None:
+            wss = (self.window_sources.get(wid) for wid in window_ids)
+        else:
+            wss = self.window_sources.values()
+        for ws in wss:
+            if ws is not None:
+                ws.set_auto_refresh_delay(delay)
+
+    def set_encoding(self, encoding, window_ids, strict=False):
         """ Changes the encoder for the given 'window_ids',
             or for all windows if 'window_ids' is None.
         """
+        if not self.ui_client:
+            return
         encoding = OLD_ENCODING_NAMES_TO_NEW.get(encoding, encoding)
         if encoding:
             if encoding not in self.encodings:
@@ -951,8 +985,9 @@ class ServerSource(object):
                 common = [e for e in self.encodings if e in self.server_encodings]
                 elog("encodings supported by both ends: %s", common)
                 if not common:
-                    raise Exception("cannot find compatible encoding between "
-                                    "client (%s) and server (%s)" % (self.encodings, self.server_encodings))
+                    log.error("cannot find compatible encoding between "
+                                    "client (%s) and server (%s)" % (", ".join(self.encodings), ", ".join(self.server_encodings)))
+                    raise ClientException("cannot find a common encoding to use")
                 encoding = common[0]
         if window_ids is not None:
             wss = [self.window_sources.get(wid) for wid in window_ids]
@@ -965,38 +1000,46 @@ class ServerSource(object):
             self.global_batch_config = self.default_batch_config.clone()
         for ws in wss:
             if ws is not None:
-                ws.set_new_encoding(encoding)
+                ws.set_new_encoding(encoding, strict)
         if not window_ids or self.encoding is None:
             self.encoding = encoding
 
     def hello(self, server_capabilities):
         capabilities = server_capabilities.copy()
-        try:
-            from xpra.sound.gstreamer_util import has_gst, add_gst_capabilities
-        except:
-            has_gst = False
-        if has_gst:
+        if self.wants_sound:
+            sound_caps = {}
             try:
-                from xpra.sound.pulseaudio_util import add_pulseaudio_capabilities
-                add_pulseaudio_capabilities(capabilities)
-                add_gst_capabilities(capabilities,
-                                     receive=self.supports_microphone, send=self.supports_speaker,
-                                     receive_codecs=self.speaker_codecs, send_codecs=self.microphone_codecs,
-                                     new_namespace=self.namespace)
-                log("sound capabilities: %s", [(k,v) for k,v in capabilities.items() if k.startswith("sound.")])
+                from xpra.sound.gstreamer_util import get_info as get_gst_info
+                sound_caps.update(get_gst_info(receive=self.supports_microphone, send=self.supports_speaker,
+                                 receive_codecs=self.speaker_codecs, send_codecs=self.microphone_codecs))
+            except ImportError:
+                pass
             except Exception, e:
                 log.error("failed to setup sound: %s", e)
-        if self.send_windows:
-            assert self.encoding
+            try:
+                from xpra.sound.pulseaudio_util import get_info as get_pa_info
+                sound_caps.update(get_pa_info())
+            except ImportError:
+                pass
+            except Exception, e:
+                log.error("failed to setup sound: %s", e)
+            if self.namespace:
+                updict(capabilities, "sound", sound_caps)
+            else:
+                capabilities.update(sound_caps)
+            log("sound capabilities: %s", sound_caps)
+        if self.wants_encodings or self.send_windows:
+            assert self.encoding, "cannot send windows/encodings without an encoding!"
             encoding = self.encoding
             if not self.generic_encodings:
                 #translate back into the legacy names:
                 encoding = NEW_ENCODING_NAMES_TO_OLD.get(encoding, encoding)
             capabilities["encoding"] = encoding
-        capabilities.update({
-                     "mmap_enabled"         : self.mmap_size>0,
-                     "auto_refresh_delay"   : self.auto_refresh_delay,
-                     })
+        if self.wants_features:
+            capabilities.update({
+                         "mmap_enabled"         : self.mmap_size>0,
+                         "auto_refresh_delay"   : self.auto_refresh_delay,
+                         })
         if self.mmap_client_token:
             capabilities["mmap_token"] = self.mmap_client_token
         if self.keyboard_config:
@@ -1004,98 +1047,120 @@ class ServerSource(object):
         self.rewrite_encoding_values(capabilities)
         self.send("hello", capabilities)
 
-    def add_info(self, info, suffix=""):
-        if self.namespace:
-            k = "server.python.version"
-        else:
-            k = "python_version"
-        info[k] = python_platform.python_version()
-        def cv(name, v):
-            info["client."+name+suffix] = v
-        def cvs(update):
-            for k,v in update.items():
-                cv(k, v)
+
+    def get_info(self):
+        info = {
+                "version"           : self.client_version or "unknown",
+                "platform_name"     : platform_name(self.client_platform, self.client_release),
+                "uuid"              : self.uuid,
+                "idle_time"         : int(time.time()-self.last_user_event),
+                "hostname"          : self.hostname,
+                "auto_refresh"      : self.auto_refresh_delay,
+                "desktop_size"      : self.desktop_size or "",
+                "connection_time"   : int(self.connection_time),
+                "elapsed_time"      : int(time.time()-self.connection_time),
+                "suspended"         : self.suspended,
+                }
+
         def addattr(k, name):
-            try:
-                v = getattr(self, name)
-                if v is not None:
-                    cv(k, v)
-            except:
-                log.warn("cannot add attribute %s / %s", k, name, exc_info=True)
-        cv("version", self.client_version or "unknown")
+            v = getattr(self, name)
+            if v is not None:
+                info[k] = v
         for x in ("type", "platform", "release", "machine", "processor", "proxy"):
-            addattr(x, "client_" + x)
-        cvs({
-             "platform_name"      : platform_name(self.client_platform, self.client_release),
-             "uuid"               : self.uuid,
-             "idle_time"          : int(time.time()-self.last_user_event),
-             "hostname"           : self.hostname,
-             "auto_refresh"       : self.auto_refresh_delay,
-             "desktop_size"       : self.desktop_size or "",
-             "connection_time"    : int(self.connection_time),
-             "elapsed_time"       : int(time.time()-self.connection_time),
-             "suspended"          : self.suspended,
+            addattr(x, "client_"+x)
+        #encoding:
+        info.update({
+             "encodings"         : self.encodings,
+             "encodings.core"    : self.core_encodings,
+             "encoding.default"  : self.default_encoding or ""
              })
-        #= time.time()
-        #self.start_time = time.time()
-        if self.screen_sizes:
-            info["client.screens" + suffix] = len(self.screen_sizes)
-            i = 0
-            for x in self.screen_sizes:
-                if type(x) not in (tuple, list):
-                    #legacy clients:
-                    cv("screen[%s]=" % i, str(x))
-                    continue
-                cv("screen[%s].display" % i, x[0])
-                if len(x)>=3:
-                    cv("screen[%s].size" % i, (x[1], x[2]))
-                if len(x)>=5:
-                    cv("screen[%s].size_mm" % i, (x[3], x[4]))
-                if len(x)>=6:
-                    monitors = x[5]
-                    j = 0
-                    for monitor in monitors:
-                        if len(monitor)>=7:
-                            cv("screen[%s].monitor[%s].name" % (i, j), monitor[0])
-                            cv("screen[%s].monitor[%s].geometry" % (i, j), monitor[1:5])
-                            cv("screen[%s].monitor[%s].size_mm" % (i, j), monitor[5:7])
-                        j += 1
-                if len(x)>=10:
-                    cv("screen[%s].workarea" % i, x[6:10])
-                i += 1
+        def up(prefix, d):
+            updict(info, prefix, d)
+        up("encoding",      self.default_encoding_options)
+        up("encoding",      self.encoding_options)
+        up("connection",    self.protocol.get_info())
+        info.update(self.get_sound_info())
+        info.update(self.get_features_info())
+        info.update(self.get_screen_info())
+        return info
+
+    def get_screen_info(self):
+        return get_screen_info(self.screen_sizes)
+
+    def get_features_info(self):
+        info = {}
+        def battr(k, name):
+            info[k] = bool(getattr(self, name))
         for prop in ("named_cursors", "server_window_resize", "share", "randr_notify",
                      "clipboard_notifications", "raw_window_icons", "system_tray", "generic_window_types",
-                     "notify_startup_complete", "namespace", "lz4"):
-            addattr("features."+prop, prop)
+                     "notify_startup_complete", "namespace", "lz4", "lzo"):
+            battr(prop, prop)
         for prop, name in {"clipboard_enabled"  : "clipboard",
                            "send_windows"       : "windows",
                            "send_cursors"       : "cursors",
                            "send_notifications" : "notifications",
                            "send_bell"          : "bell"}.items():
-            addattr("features."+name, prop)
-        #encoding:
-        cvs({
-             "encodings"         : self.encodings,
-             "encodings.core"    : self.core_encodings,
-             "encoding.default"  : self.default_encoding or ""
-             })
-        for k,v in self.default_encoding_options.items():
-            cv("encoding.%s" % k, v)
-        for k,v in self.encoding_options.items():
-            cv("encoding.%s" % k, v)
-        def get_sound_info(supported, prop):
+            battr(name, prop)
+        return info
+
+    def get_sound_info(self):
+        def sound_info(supported, prop):
             if not supported:
                 return {"state" : "disabled"}
             if prop is None:
                 return {"state" : "inactive"}
             return prop.get_info()
-        #sound:
+        info = {}
         for prop in ("pulseaudio_id", "pulseaudio_server"):
-            addattr(prop, prop)
-        for k,v in get_sound_info(self.supports_speaker, self.sound_source).items():
-            cv("speaker.%s" % k, v)
-        for k,v in get_sound_info(self.supports_microphone, self.sound_sink).items():
-            cv("microphone.%s" % k, v)
+            v = getattr(self, prop)
+            if v is not None:
+                info[prop] = v
+        for k,v in sound_info(self.supports_speaker, self.sound_source).items():
+            info["speaker.%s" % k] = v
+        for k,v in sound_info(self.supports_microphone, self.sound_sink).items():
+            info["microphone.%s" % k] = v
+        return info
+
+    def get_window_info(self, window_ids=[]):
+        """
+            Adds encoding and window specific information
+        """
+        info = {
+            "damage.compression_queue.size.current" : self.compression_work_queue.qsize(),
+            "damage.packet_queue.size.current"      : len(self.packet_queue),
+            }
+        qpixels = [x[2] for x in list(self.packet_queue)]
+        add_list_stats(info, "packet_queue_pixels",  qpixels)
+        if len(qpixels)>0:
+            info["packet_queue_pixels.current"] = qpixels[-1]
+
+        info.update(self.statistics.get_info())
+
+        if len(window_ids)>0:
+            total_pixels = 0
+            total_time = 0.0
+            in_latencies, out_latencies = [], []
+            for wid in window_ids:
+                ws = self.window_sources.get(wid)
+                if ws is None:
+                    continue
+                #per-window source stats:
+                updict(info, "window[%i]" % wid, ws.get_info())
+                #collect stats for global averages:
+                for _, pixels, _, _, encoding_time in list(ws.statistics.encoding_stats):
+                    total_pixels += pixels
+                    total_time += encoding_time
+                in_latencies += [x*1000 for _, _, _, x in list(ws.statistics.damage_in_latency)]
+                out_latencies += [x*1000 for _, _, _, x in list(ws.statistics.damage_out_latency)]
+            v = 0
+            if total_time>0:
+                v = int(total_pixels / total_time)
+            info["encoding.pixels_encoded_per_second"] = v
+            add_list_stats(info, "damage.in_latency",  in_latencies, show_percentile=[9])
+            add_list_stats(info, "damage.out_latency",  out_latencies, show_percentile=[9])
+        updict(info, "batch", self.global_batch_config.get_info())
+        return info
+
 
     def send_info_response(self, info):
         self.rewrite_encoding_values(info)
@@ -1117,7 +1182,7 @@ class ServerSource(object):
         if len(replace)==0:
             return
         #filter only "encodings.*" keys:
-        for k in [x for x in d.keys() if x=="encodings" or x.startswith("encodings.")]:
+        for k in (x for x in d.keys() if x=="encodings" or x.startswith("encodings.")):
             v = d.get(k)
             if type(v) not in (list, tuple):
                 continue
@@ -1133,13 +1198,28 @@ class ServerSource(object):
                 log("rewrite_encoding_values for key '%s': %s replaced by %s", k, l, newlist)
 
 
+    def send_server_event(self, *args):
+        if self.wants_events:
+            self.send("server-event", *args)
+
+
     def send_clipboard_enabled(self, reason=""):
         self.send("set-clipboard-enabled", self.clipboard_enabled, reason)
 
     def send_clipboard(self, packet):
         if not self.clipboard_enabled or self.suspended:
             return
-        self.send(*packet)
+        #call compress_clibboard via the work queue:
+        self.compression_work_queue.put((self.compress_clipboard, packet))
+
+    def compress_clipboard(self, packet):
+        #Note: this runs in the 'encode' thread!
+        packet = list(packet)
+        for i in range(len(packet)):
+            v = packet[i]
+            if type(v)==Uncompressed:
+                packet[i] = self.compressed_wrapper(v.datatype, v.data)
+        self.queue_packet(packet)
 
 
     def pointer_grab(self, wid):
@@ -1151,27 +1231,50 @@ class ServerSource(object):
             self.send("pointer-ungrab", wid)
 
 
-    def send_cursor(self, cursor_data, sizes):
+    def compressed_wrapper(self, datatype, data):
+        if self.zlib or self.lz4 or self.lzo:
+            return compressed_wrapper(datatype, data, zlib=self.zlib, lz4=self.lz4, lzo=self.lzo, can_inline=False)
+        #we can't compress, so at least avoid warnings in the protocol layer:
+        return Compressed("raw %s" % datatype, data, can_inline=True)
+
+    def send_cursor(self, get_cursor_data_cb):
         if not self.send_cursors or self.suspended:
             return
-        self.cursor_data = cursor_data
-        self.cursor_sizes = sizes
+        def do_send_cursor():
+            self.send_cursor_pending = False
+            cursor_data, cursor_sizes = get_cursor_data_cb()
+            if cursor_data:
+                if self.last_cursor_sent and self.last_cursor_sent==cursor_data[:8]:
+                    cursorlog("do_send_cursor(..) cursor identical to the last one we sent, nothing to do")
+                    return
+                self.last_cursor_sent = cursor_data[:8]
+                w, h, _xhot, _yhot, serial, pixels = cursor_data[2:8]
+                #compress pixels if needed:
+                if pixels is not None:
+                    #convert bytearray to string:
+                    cpixels = str(pixels)
+                    if len(cpixels)>=256:
+                        cpixels = self.compressed_wrapper("cursor", pixels)
+                        cursorlog("do_send_cursor(..) pixels=%s ", cpixels)
+                    cursor_data[7] = cpixels
+                cursorlog("do_send_cursor(..) %sx%s cursor %s with delay=%s", w, h, serial, delay)
+                #prepare packet format:
+                if not self.named_cursors:
+                    #old versions have limited support (no cursor names, no sizes):
+                    args = cursor_data[:8]
+                else:
+                    args = list(cursor_data[:9]) + list(cursor_sizes)
+            else:
+                cursorlog("do_send_cursor(..) sending empty cursor with delay=%s", delay)
+                args = [""]
+                self.last_cursor_sent = None
+            self.send("cursor", *args)
+        #if not pending already, schedule it:
         if not self.send_cursor_pending:
             self.send_cursor_pending = True
             delay = max(10, int(self.global_batch_config.delay/4))
-            self.timeout_add(delay, self.do_send_cursor)
+            self.timeout_add(delay, do_send_cursor)
 
-    def do_send_cursor(self):
-        self.send_cursor_pending = False
-        if self.cursor_data:
-            if not self.named_cursors:
-                #old versions have limited support (no cursor names, no sizes):
-                args = self.cursor_data[:8]
-            else:
-                args = list(self.cursor_data[:9]) + list(self.cursor_sizes)
-            self.send("cursor", *args)
-        else:
-            self.send("cursor", "")
 
     def bell(self, wid, device, percent, pitch, duration, bell_class, bell_id, bell_name):
         if not self.send_bell or self.suspended:
@@ -1207,7 +1310,7 @@ class ServerSource(object):
         timeout = 60
         def check_echo_timeout(*args):
             if self.last_ping_echoed_time<now_ms and not self.is_closed():
-                self.disconnect("client ping timeout, - waited %s seconds without a response" % timeout)
+                self.disconnect(CLIENT_PING_TIMEOUT, "waited %s seconds without a response" % timeout)
         self.timeout_add(timeout*1000, check_echo_timeout)
 
     def process_ping(self, time_to_echo):
@@ -1260,11 +1363,12 @@ class ServerSource(object):
                 self.send("window-metadata", wid, metadata)
 
     def can_send_window(self, window):
-        if not self.send_windows and not window.is_tray():
-            return  False
-        if window.is_tray() and not self.system_tray:
-            return  False
-        return True
+        if self.send_windows and self.system_tray:
+            #common case shortcut
+            return True
+        if window.is_tray():
+            return self.system_tray
+        return self.send_windows
 
     def new_tray(self, wid, window, w, h):
         assert window.is_tray()
@@ -1297,7 +1401,7 @@ class ServerSource(object):
             w, h, pixel_format, pixel_data = make_window_icon(surf, ("png" in self.encodings))
             assert pixel_format in ("premult_argb32", "png")
             if pixel_format=="premult_argb32":
-                data = compressed_wrapper("argb32", pixel_data)
+                data = self.compressed_wrapper("argb32", pixel_data)
             else:
                 data = Compressed("png", pixel_data)
             self.send("window-icon", wid, w, h, pixel_format, data)
@@ -1354,45 +1458,6 @@ class ServerSource(object):
             del self.window_sources[wid]
             ws.cleanup()
 
-    def add_stats(self, info, window_ids=[], suffix=""):
-        """
-            Adds most of the statistics available to the 'info' dict passed in.
-            This is used by server.py to provide those statistics to clients
-            via the 'xpra info' command.
-        """
-        info["damage.data_queue.size%s.current" % suffix] = self.damage_data_queue.qsize()
-        info["damage.packet_queue.size%s.current" % suffix] = len(self.damage_packet_queue)
-        qpixels = [x[2] for x in list(self.damage_packet_queue)]
-        add_list_stats(info, "damage_packet_queue_pixels"+suffix,  qpixels)
-        if len(qpixels)>0:
-            info["damage_packet_queue_pixels%s.current" % suffix] = qpixels[-1]
-
-        self.protocol.add_stats(info, prefix="client.connection.", suffix=suffix)
-        self.statistics.add_stats(info, suffix=suffix)
-        if len(window_ids)>0:
-            total_pixels = 0
-            total_time = 0.0
-            in_latencies = []
-            out_latencies = []
-            for wid in window_ids:
-                ws = self.window_sources.get(wid)
-                if ws is None:
-                    continue
-                #per-window source stats:
-                ws.add_stats(info, suffix=suffix)
-                #collect stats for global averages:
-                for _, pixels, _, _, encoding_time in list(ws.statistics.encoding_stats):
-                    total_pixels += pixels
-                    total_time += encoding_time
-                in_latencies += [x*1000 for _, _, _, x in list(ws.statistics.damage_in_latency)]
-                out_latencies += [x*1000 for _, _, _, x in list(ws.statistics.damage_out_latency)]
-            v = 0
-            if total_time>0:
-                v = int(total_pixels / total_time)
-            info["encoding.pixels_encoded_per_second"+suffix] = v
-            add_list_stats(info, "damage.in_latency",  in_latencies, show_percentile=[9])
-            add_list_stats(info, "damage.out_latency",  out_latencies, show_percentile=[9])
-        self.global_batch_config.add_stats(info, "", suffix)
 
     def set_min_quality(self, min_quality):
         for ws in list(self.window_sources.values()):
@@ -1410,12 +1475,6 @@ class ServerSource(object):
         for ws in list(self.window_sources.values()):
             ws.set_speed(speed)
 
-    def full_quality_refresh(self, wid, window, damage_options):
-        if not self.can_send_window(window):
-            return
-        ws = self.window_sources.get(wid)
-        if ws:
-            ws.full_quality_refresh(window, damage_options)
 
     def refresh(self, wid, window, opts):
         if not self.can_send_window(window):
@@ -1438,6 +1497,7 @@ class ServerSource(object):
             log("batch config updated for window %s: %s", wid, ws.batch_config)
 
     def set_client_properties(self, wid, window, new_client_properties):
+        assert self.send_windows
         ws = self.make_window_source(wid, window)
         ws.set_client_properties(new_client_properties)
 
@@ -1463,12 +1523,11 @@ class ServerSource(object):
             #or for transparent windows (if the client supports transparency) since video doesn't do alpha (yet?)
             if not window.is_tray() and (not window.has_alpha() or not self.supports_transparency):
                 wts = get_generic_window_type(window, strip_net=True)
-                video_window_types = ("NORMAL", "DIALOG", "DESKTOP")
-                if len([x for x in wts if x in video_window_types])>0:
+                if set(("NORMAL", "DIALOG", "DESKTOP")).intersection(wts):
                     wclass = WindowVideoSource
 
             ws = wclass(self.idle_add, self.timeout_add, self.source_remove,
-                              self.queue_damage, self.queue_packet,
+                              self.queue_size, self.queue_damage, self.queue_packet,
                               self.statistics,
                               wid, window, batch_config, self.auto_refresh_delay,
                               self.video_helper,
@@ -1517,23 +1576,27 @@ class ServerSource(object):
 #
 # Methods used by WindowSource:
 #
+    def queue_size(self):
+        return self.compression_work_queue.qsize()
+
     def queue_damage(self, *fn_and_args):
         """
-            This is used by WindowSource to queue damage processing to be done in the 'data_to_packet' thread.
-            The 'encode_and_send_cb' will then add the resulting packet to the 'damage_packet_queue' via 'queue_packet'.
+            This is used by WindowSource to queue damage processing to be done in the 'encode' thread.
+            The 'encode_and_send_cb' will then add the resulting packet to the 'packet_queue' via 'queue_packet'.
         """
-        self.statistics.damage_data_qsizes.append((time.time(), self.damage_data_queue.qsize()))
-        self.damage_data_queue.put(fn_and_args)
+        self.statistics.compression_work_qsizes.append((time.time(), self.compression_work_queue.qsize()))
+        self.compression_work_queue.put(fn_and_args)
 
-    def queue_packet(self, packet, wid, pixels, start_send_cb, end_send_cb):
+    def queue_packet(self, packet, wid=0, pixels=0, start_send_cb=None, end_send_cb=None):
         """
-            Add a new 'draw' packet to the 'damage_packet_queue'.
+            Add a new 'draw' packet to the 'packet_queue'.
             Note: this code runs in the non-ui thread
         """
         now = time.time()
-        self.statistics.damage_packet_qsizes.append((now, len(self.damage_packet_queue)))
-        self.statistics.damage_packet_qpixels.append((now, wid, sum([x[2] for x in list(self.damage_packet_queue) if x[1]==wid])))
-        self.damage_packet_queue.append((packet, wid, pixels, start_send_cb, end_send_cb))
+        self.statistics.packet_qsizes.append((now, len(self.packet_queue)))
+        if wid>0:
+            self.statistics.damage_packet_qpixels.append((now, wid, sum(x[2] for x in list(self.packet_queue) if x[1]==wid)))
+        self.packet_queue.append((packet, wid, pixels, start_send_cb, end_send_cb))
         p = self.protocol
         if p:
             p.source_has_more()
@@ -1541,15 +1604,15 @@ class ServerSource(object):
 #
 # The damage packet thread loop:
 #
-    def data_to_packet(self):
+    def encode_loop(self):
         """
             This runs in a separate thread and calls all the function callbacks
-            which are added to the 'damage_data_queue'.
+            which are added to the 'compression_work_queue'.
             Must run until we hit the end of queue marker,
             to ensure all the queued items get called.
         """
         while True:
-            fn_and_args = self.damage_data_queue.get(True)
+            fn_and_args = self.compression_work_queue.get(True)
             if fn_and_args is None:
                 return              #empty marker
             try:
