@@ -279,9 +279,6 @@ class BaseWindowModel(AutoPropGObjectMixin, gobject.GObject):
                        "Is the window a system tray icon", "",
                        False,
                        gobject.PARAM_READABLE),
-        "scaling" : (gobject.TYPE_PYOBJECT,
-                       "Application requested scaling as a fraction (pair of numbers)", "",
-                       gobject.PARAM_READWRITE),
         "role" : (gobject.TYPE_PYOBJECT,
                           "The window's role (ICCCM session management)", "",
                           gobject.PARAM_READABLE),
@@ -484,21 +481,10 @@ class BaseWindowModel(AutoPropGObjectMixin, gobject.GObject):
         self._internal_set_property("xid", self.client_window.xid)
         self._internal_set_property("pid", pget("_NET_WM_PID", "u32") or -1)
         self._internal_set_property("role", pget("WM_WINDOW_ROLE", "latin1"))
-        for mutable in ["WM_NAME", "_NET_WM_NAME", "_NET_WM_WINDOW_OPACITY", "_XPRA_SCALING"]:
+        for mutable in ["WM_NAME", "_NET_WM_NAME", "_NET_WM_WINDOW_OPACITY"]:
             self._call_property_handler(mutable)
 
 
-    def _handle_scaling(self):
-        scaling = self.prop_get("_XPRA_SCALING", "u32", raise_xerrors=False)
-        scaling_v, scaling_u = 1, 1
-        if scaling>0:
-            scaling_v = scaling & 0xFFFF
-            scaling_u = (scaling >> 16) & 0xFFFF
-            self._internal_set_property("scaling", (scaling_v, scaling_u))
-        else:
-            self._internal_set_property("scaling", None)
-    _property_handlers["_XPRA_SCALING"] = _handle_scaling
-
     def _handle_opacity_change(self):
         opacity = self.prop_get("_NET_WM_WINDOW_OPACITY", "u32", True) or -1
         self._internal_set_property("opacity", opacity)
@@ -855,7 +841,7 @@ class WindowModel(BaseWindowModel):
                                             wclass=gtk.gdk.INPUT_OUTPUT,
                                             event_mask=gtk.gdk.PROPERTY_CHANGE_MASK,
                                             title = "CorralWindow-%#x" % self.client_window.xid)
-        log("setup() corral_window=%s", self.corral_window)
+        log("setup() corral_window=%#x", self.corral_window.xid)
         X11Window.substructureRedirect(self.corral_window.xid)
         add_event_receiver(self.corral_window, self)
 
@@ -928,7 +914,7 @@ class WindowModel(BaseWindowModel):
         # the client is naughty and tried to map their window which is
         # currently not displayed.  In either case, we should just ignore the
         # request.
-        pass
+        log("do_child_map_request_event(%s)", event)
 
     def do_xpra_unmap_event(self, event):
         if event.delivered_to is self.corral_window or self.corral_window is None:
@@ -946,7 +932,7 @@ class WindowModel(BaseWindowModel):
         # Also, if we receive a *synthetic* UnmapNotify event, that always
         # means that the client has withdrawn the window (even if it was not
         # mapped in the first place) -- ICCCM section 4.1.4.
-        log("Client window unmapped")
+        log("do_xpra_unmap_event(%s) client window unmapped", event)
         if event.send_event or event.serial != self.startup_unmap_serial:
             self.unmanage()
 
