@@ -47,15 +47,18 @@ import xpra
 from xpra.util import nonl
 from xpra.os_util import StringIOClass
 from xpra.x11.x11_server_base import X11ServerBase, mouselog
-from xpra.net.protocol import compressed_wrapper, Compressed
+from xpra.net.compression import Compressed
 
 
 class DesktopManager(gtk.Widget):
     def __init__(self):
+        self._models = {}
         gtk.Widget.__init__(self)
         self.set_property("can-focus", True)
         self.set_flags(gtk.NO_WINDOW)
-        self._models = {}
+
+    def __repr__(self):
+        return "DesktopManager(%s)" % len(self._models)
 
     ## For communicating with the main WM:
 
@@ -208,9 +211,8 @@ class XpraServer(gobject.GObject, X11ServerBase):
         #cursor:
         self.default_cursor_data = None
         self.last_cursor_serial = None
+        self.last_cursor_data = None
         self.send_cursor_pending = False
-        self.cursor_data = None
-        self.cursor_sizes = None
         def get_default_cursor():
             self.default_cursor_data = X11Keyboard.get_cursor_image()
             cursorlog("get_default_cursor=%s", self.default_cursor_data)
@@ -218,19 +220,20 @@ class XpraServer(gobject.GObject, X11ServerBase):
         self._wm.enableCursors(True)
 
 
-    def make_hello(self):
-        capabilities = X11ServerBase.make_hello(self)
-        capabilities["window.raise"] = True
-        capabilities["window.resize-counter"] = True
-        capabilities["pointer.grabs"] = True
+    def make_hello(self, source):
+        capabilities = X11ServerBase.make_hello(self, source)
+        if source.wants_features:
+            capabilities["window.raise"] = True
+            capabilities["window.resize-counter"] = True
+            capabilities["pointer.grabs"] = True
         return capabilities
 
 
     def do_get_info(self, proto, server_sources, window_ids):
         info = X11ServerBase.do_get_info(self, proto, server_sources, window_ids)
-        log("do_get_info: adding cursor=%s", self.cursor_data)
+        log("do_get_info: adding cursor=%s", self.last_cursor_data)
         #copy to prevent race:
-        cd = self.cursor_data
+        cd = self.last_cursor_data
         if cd is None:
             info["cursor"] = "None"
         else:
@@ -330,13 +333,13 @@ class XpraServer(gobject.GObject, X11ServerBase):
             if X11Window.is_override_redirect(window.xid) and X11Window.is_mapped(window.xid):
                 self._add_new_or_window(window)
 
-    def send_windows_and_cursors(self, ss):
+    def send_windows_and_cursors(self, ss, sharing=False):
         # We send the new-window packets sorted by id because this sorts them
         # from oldest to newest -- and preserving window creation order means
         # that the earliest override-redirect windows will be on the bottom,
         # which is usually how things work.  (I don't know that anyone cares
         # about this kind of correctness at all, but hey, doesn't hurt.)
-        windowlog("send_windows_and_cursors(%s) will send: %s", ss, self._id_to_window)
+        windowlog("send_windows_and_cursors(%s, %s) will send: %s", ss, sharing, self._id_to_window)
         for wid in sorted(self._id_to_window.keys()):
             window = self._id_to_window[wid]
             if not window.is_managed():
@@ -351,7 +354,7 @@ class XpraServer(gobject.GObject, X11ServerBase):
                 if ss.system_tray:
                     ss.new_tray(wid, window, w, h)
                     ss.damage(wid, window, 0, 0, w, h)
-                else:
+                elif not sharing:
                     #park it outside the visible area
                     window.move_resize(-200, -200, w, h)
             elif window.is_OR():
@@ -362,7 +365,8 @@ class XpraServer(gobject.GObject, X11ServerBase):
                 ss.damage(wid, window, 0, 0, w, h)
             else:
                 #code more or less duplicated from send_new_window_packet:
-                self._desktop_manager.hide_window(window)
+                if not sharing:
+                    self._desktop_manager.hide_window(window)
                 x, y, w, h = self._desktop_manager.window_geometry(window)
                 wprops = self.client_properties.get("%s|%s" % (wid, ss.uuid))
                 ss.new_window("new-window", wid, window, x, y, w, h, wprops)
@@ -370,7 +374,7 @@ class XpraServer(gobject.GObject, X11ServerBase):
         display = gtk.gdk.display_get_default()
         self.cursor_sizes = display.get_default_cursor_size(), display.get_maximal_cursor_size()
         cursorlog("cursor_sizes=%s", self.cursor_sizes)
-        ss.send_cursor(self.cursor_data, self.cursor_sizes)
+        ss.send_cursor(self.get_cursor_data)
 
 
     def _new_window_signaled(self, wm, window):
@@ -486,34 +490,27 @@ class XpraServer(gobject.GObject, X11ServerBase):
             return
         cursorlog("cursor_event: %s", event)
         self.last_cursor_serial = event.cursor_serial
-        if not self.send_cursor_pending:
-            self.send_cursor_pending = True
-            gobject.timeout_add(10, self.send_cursor)
-
-    def send_cursor(self):
-        self.send_cursor_pending = False
-        self.cursor_data = X11Keyboard.get_cursor_image()
-        display = gtk.gdk.display_get_default()
-        self.cursor_sizes = display.get_default_cursor_size(), display.get_maximal_cursor_size()
-        if self.cursor_data is not None:
-            pixels = self.cursor_data[7]
-            cursorlog("send_cursor() cursor=%s", self.cursor_data[:7]+["%s bytes" % len(pixels)]+self.cursor_data[8:])
-            if self.default_cursor_data is not None and str(pixels)==str(self.default_cursor_data[7]):
-                cursorlog("send_cursor(): default cursor - clearing it")
-                self.cursor_data = None
-            elif pixels is not None:
-                #convert bytearray to string:
-                pixels = str(pixels)
-                if len(pixels)<64:
-                    self.cursor_data[7] = pixels
-                else:
-                    self.cursor_data[7] = compressed_wrapper("cursor", pixels)
-        else:
-            cursorlog("send_cursor() failed to get cursor image")
         for ss in self._server_sources.values():
-            ss.send_cursor(self.cursor_data, self.cursor_sizes)
+            ss.send_cursor(self.get_cursor_data)
         return False
 
+    def get_cursor_data(self):
+        #must be called from the UI thread!
+        cursor_data = X11Keyboard.get_cursor_image()
+        if cursor_data is None:
+            cursorlog("get_cursor_data() failed to get cursor image")
+            return None, []
+        self.last_cursor_data = cursor_data
+        pixels = self.last_cursor_data[7]
+        cursorlog("get_cursor_data() cursor=%s", cursor_data[:7]+["%s bytes" % len(pixels)]+cursor_data[8:])
+        if self.default_cursor_data is not None and str(pixels)==str(self.default_cursor_data[7]):
+            cursorlog("get_cursor_data(): default cursor - clearing it")
+            cursor_data = None
+        display = gtk.gdk.display_get_default()
+        cursor_sizes = display.get_default_cursor_size(), display.get_maximal_cursor_size()
+        return (cursor_data, cursor_sizes)
+
+
     def _bell_signaled(self, wm, event):
         log("bell signaled on window %s", event.window.xid)
         if not self.bell:
@@ -667,9 +664,10 @@ class XpraServer(gobject.GObject, X11ServerBase):
 
     def _process_configure_window(self, proto, packet):
         wid, x, y, w, h = packet[1:6]
-        client_properties = {}
         if len(packet)>=7:
             client_properties = packet[6]
+        else:
+            client_properties = {}
         resize_counter = 0
         if len(packet)>=8:
             resize_counter = packet[7]
