@@ -143,11 +143,12 @@ class X11ServerBase(GTKServerBase):
             X11Keyboard.set_key_repeat_rate(500, 30)
             keylog("keyboard repeat disabled")
 
-    def make_hello(self):
-        capabilities = GTKServerBase.make_hello(self)
+    def make_hello(self, source):
+        capabilities = GTKServerBase.make_hello(self, source)
         capabilities["server_type"] = "Python/gtk/x11"
-        capabilities["resize_screen"] = self.randr
-        capabilities["force_ungrab"] = True
+        if source.wants_features:
+            capabilities["resize_screen"] = self.randr
+            capabilities["force_ungrab"] = True
         return capabilities
 
     def do_get_info(self, proto, server_sources, window_ids):
@@ -381,7 +382,7 @@ class X11ServerBase(GTKServerBase):
             return oldconf is not None
         if not self.fake_xinerama:
             return delfile(None)
-        ui_client_count = len([True for s in self._server_sources.values() if s.send_windows])
+        ui_client_count = len([True for s in self._server_sources.values() if s.ui_client])
         if ui_client_count!=1:
             return delfile("fakeXinerama can only be enabled for a single client (found %s)", ui_client_count)
         source = self._server_sources.values()[0]
